package compiler;

import java.io.*;
import java.net.*;
import java.util.Arrays;

import lowLevel.eval.Eval;
import lowLevel.generate.Generate;
import lowLevel.producer.JavaCompilationError;

import org.antlr.runtime.*;

import primitiveOperation.CircularityError;
import primitiveOperation.TranslateExpression;
import structure.antlrGenerated.MFJGrammarLexer;
import structure.antlrGenerated.MFJGrammarParser;
import structure.integrity.IntegrityChecker;
import structure.program.ClassTable;
import structure.resource.GreenResource;
import tools.*;
import tools.DataStructures.WrapperException;
import visitors.VisitorTranslateZero;

public class Mfjc {
  
  public static void print(Object o) {
    System.out.print(o);
  }
  
  public static void setDir(File f, ClassLoader c) {
    FileManager.createNew(f);
    dir = f;
    if (c == null) try {
      c = new URLClassLoader(new URL[]{dir.toURI().toURL()}, ClassLoader.getSystemClassLoader());
    } catch (MalformedURLException e) {
      Assertions.userErrorAssert("", e);
    }
    ucl = c;
  }
  public static File getDir() {
    return dir;
  }
  public static ClassLoader getLoader() {
    return ucl;
  }
  private static File dir;
  private static ClassLoader ucl;
  public static StringBuffer programCode = new StringBuffer("programCode");
  public static void main(String[] args) throws Throwable {
    try {
      if (args.length != 2) {
        System.out.println("use with inputName outputFolder, you give " + Arrays.asList(args));
        return;
      }
      setDir(new File(args[1]), null);
      Eval.setDir(dir, getLoader());
      FileManager.createNew(Mfjc.getDir());
      FileManager.deleteDir(new File(new File(args[1]), "" + Eval.packageName));
      FileManager.deleteDir(new File(new File(args[1]), "" + programCode));
      compileFile(new File(args[0]));
      System.exit(0);
    } catch (Throwable t) {
      ReportingHelpers.printAllAbout(t);
      System.exit(1);
    }
  }
  
  public static ClassTable parseFile(File file) throws FileNotFoundException, RecognitionException, IOException {
    return parseStream(new FileReader(file));
  }
  public static ClassTable parseString(String code) throws FileNotFoundException, RecognitionException, IOException {
    return parseStream(new StringReader(code));
  }
  public static ClassTable parseStream(Reader r) throws RecognitionException, IOException {
    ANTLRReaderStream input = new ANTLRReaderStream(r);
    MFJGrammarLexer lexer = new MFJGrammarLexer(input);
    CommonTokenStream tokens = new CommonTokenStream(lexer);
    MFJGrammarParser parser = new MFJGrammarParser(tokens);
    return (ClassTable) parser.classTable();
  }
  
  public static void compileFile(File file) throws IOException, RecognitionException, JavaCompilationError, CircularityError {
    compileStream(new FileReader(file));
  }
  public static void compileString(String code) throws IOException, RecognitionException, JavaCompilationError, CircularityError {
    compileStream(new StringReader(code));
  }
  public static void compileStream(Reader r) throws IOException, RecognitionException, JavaCompilationError, CircularityError {
    compileClassTable(parseStream(r));
  }
  public static void compileClassTable(ClassTable ct) throws JavaCompilationError, CircularityError {
    //classTable = ct;
    try {
      printClassTableStatus(ct);
      while (ct.keySet(GreenResource.class).size() < ct.keySet().size()) {
        strategy.MakeOneStep.run(ct);
        printClassTableStatus(ct);
      }
      generateMain(ct);
    } catch (WrapperException e) {
      if (e.wrappedElement instanceof RuntimeException) throw (RuntimeException) e.wrappedElement;
      if (e.wrappedElement instanceof Error) throw (Error) e.wrappedElement;
      throw Assertions.codeNotReachable();
    }
  }
  
  public static void generateMain(ClassTable ct) throws JavaCompilationError {
    assert ct.values().size() == ct.values(GreenResource.class).size();
    String mainName = IntegrityChecker.forbiddenPrefix + VisitorTranslateZero.generationNumber + "Main";
    StringBuffer mainClass = TranslateExpression.runMain(mainName, ct.mainExpression);
    Generate.generateSingleClass(mainName, mainClass);
  }
  private static void printClassTableStatus(ClassTable ct) {
  //System.out.println("RED: " + ct.keySet(Resource.class));
  //System.out.println("MetaYELLOW: " + ct.keySet(MetaYellowResource.class));
  //System.out.println("NormalizedYELLOW: " + ct.keySet(NormalizedYellowResource.class));
  //System.out.println("GREEN: " + ct.keySet(GreenResource.class));
  }
}
