package graphicApplication;

import java.awt.*;

import javax.swing.JTextPane;
import javax.swing.border.AbstractBorder;

public class LineNumberedBorder extends AbstractBorder {
  public static enum Side {
    LEFT_SIDE, RIGHT_SIZE
  }
  public static enum Justify {
    LEFT_JUSTIFY, RIGHT_JUSTIFY
  }
  public static final Side LEFT_SIDE = Side.LEFT_SIDE;
  public static final Side RIGHT_SIDE = Side.RIGHT_SIZE;
  public static final Justify LEFT_JUSTIFY = Justify.LEFT_JUSTIFY;
  public static final Justify RIGHT_JUSTIFY = Justify.RIGHT_JUSTIFY;
  
  private Justify lineNumberJustification;
  private Side location;
  private Component graphixFix;
  public LineNumberedBorder(Side location, Justify justify, Component gf) {
    setLocation(location);
    setLineNumberJustification(justify);
    graphixFix = gf;
  }
  
  @Override
  public Insets getBorderInsets(Component c) {
    return getBorderInsets(c, new Insets(0, 0, 0, 0));
  }
  
  /**
   *  This modifies the insets, by adding space for the line number on the
   *  left. Should be modified to add space on the right, depending upon
   *  Locale.
   *
   *@param  c       Description of the Parameter
   *@param  insets  Description of the Parameter
   *@return         The borderInsets value
   */
  @Override
  public Insets getBorderInsets(Component c, Insets insets) {
    // if c is not a JTextArea...nothing is done...
    if (c instanceof JTextPane) {
      int width = lineNumberWidth((JTextPane) c);
      if (location == LEFT_SIDE) {
        insets.left = width;
      } else {
        insets.right = width;
      }
    }
    return insets;
  }
  
  public Justify getLineNumberJustification() {
    return lineNumberJustification;
  }
  
  public void setLineNumberJustification(Justify justify) {
    lineNumberJustification = justify;
  }
  
  public Side getLocation() {
    return location;
  }
  
  public void setLocation(Side loc) {
    location = loc;
  }
  
  /**
   *  Returns the width, in pixels, of the maximum line number, plus a trailing
   *  space.
   *
   *@param  textArea  Description of the Parameter
   *@return           Description of the Return Value
   */
  private int lineNumberWidth(JTextPane textArea) {
    //
    // note: should this be changed to use all nines for the lineCount?
    // for example, if the number of rows is 111...999 could be wider
    // (in pixels) in a proportionally spaced font...
    //
    ////int lineCount = Math.max(textArea.getRows(), textArea.getLineCount() + 1);
    //TODO:
    int lineCount = 99999;
    return textArea.getFontMetrics(textArea.getFont()).stringWidth(lineCount + " ");
  }
  
  //
  // NOTE: This method is called every time the cursor blinks...
  //       so...optimize (later and if possible) for speed...
  //
  @Override
  public void paintBorder(Component c, Graphics g, int x, int y, int width, int height) {
    java.awt.Rectangle clip = g.getClipBounds();
    
    FontMetrics fm = g.getFontMetrics();
    //FontMetrics fm = graphixFix.getGraphics().getFontMetrics();
    //FontMetrics fm=new FontMetrics(new Font("Monospaced",Font.PLAIN,12));
    int fontHeight = fm.getHeight() + 1;
    
    // starting location at the "top" of the page...
    // y is the starting baseline for the font...
    // should "font leading" be applied?
    int ybaseline = y + fm.getAscent();
    
    //
    // now determine if it is the "top" of the page...or somewhere else
    //
    int startingLineNumber = (clip.y / fontHeight) + 1;
    
    //
    // use any one of the following if's:
    //
    //    if (startingLineNumber != 1)
    if (ybaseline < clip.y) {
      //
      // not within the clip rectangle...move it...
      // determine how many fontHeight's there are between
      // y and clip.y...then add that many fontHeights
      //
      ybaseline = y + startingLineNumber * fontHeight - (fontHeight - fm.getAscent());
    }
    
    //
    // options:
    // . write the number rows in the document (current)
    // . write the number of existing lines in the document (to do)
    //   see getLineCount()
    //
    
    // determine which the "drawing" should end...
    // add fontHeight: make sure...part of the line number is drawn
    //
    // could also do this by determining what the last line
    // number to draw.
    // then the "while" loop whould change accordingly.
    //
    //int yend = y + clip.height + fontHeight;
    //int yend = ybaseline + height + fontHeight; // original
    int yend = ybaseline + height;
    if (yend > (y + height)) {
      yend = y + height;
    }
    
    JTextPane jta = (JTextPane) c;
    int lineWidth = lineNumberWidth(jta);
    
    // base x position of the line number
    int lnxstart = x;
    if (location == LEFT_SIDE) {
      // x (LEFT) or (x + lineWidth) (RIGHT)
      // (depends upon justification)
      if (lineNumberJustification == LEFT_JUSTIFY) {
        lnxstart = x;
      } else {
        // RIGHT JUSTIFY
        lnxstart = x + lineWidth;
      }
    } else {
      // RIGHT SIDE
      // (y + width) - lineWidth (LEFT) or (y + width) (RIGHT)
      // (depends upon justification)
      if (lineNumberJustification == LEFT_JUSTIFY) {
        lnxstart = (y + width) - lineWidth;
      } else {
        // RIGHT JUSTIFY
        lnxstart = (y + width);
      }
    }
    
    g.setColor(c.getForeground());
    //
    // loop until out of the "visible" region...
    //
    /////int length = ("" + Math.max(jta.getRows(), jta.getLineCount() + 1)).length();
    //TODO
    int length = 99999;
    while (ybaseline < yend) {
      //
      // options:
      // . left justify the line numbers
      // . right justify the line numbers
      //
      
      if (lineNumberJustification == LEFT_JUSTIFY) {
        g.drawString(startingLineNumber + " ", lnxstart, ybaseline);
      } else {
        // right justify
        String label = padLabel(startingLineNumber, length, true);
        g.drawString(label, lnxstart - fm.stringWidth(label), ybaseline);
      }
      
      ybaseline += fontHeight;//TODO
      startingLineNumber++;
    }
  }
  // paintComponent
  
  /**
   *  Create the string for the line number. NOTE: The <tt>length</tt> param
   *  does not include the <em>optional</em> space added after the line number.
   *
   *@param  lineNumber  to stringize
   *@param  length      the length desired of the string
   *@param  addSpace    Description of the Parameter
   *@return             the line number for drawing
   */
  private static String padLabel(int lineNumber, int length, boolean addSpace) {
    StringBuffer buffer = new StringBuffer();
    buffer.append(lineNumber);
    for (int count = (length - buffer.length()); count > 0; count--) {
      buffer.insert(0, ' ');
    }
    if (addSpace) {
      buffer.append(' ');
    }
    return buffer.toString();
  }
}
// LineNumberedBorder