package graphicApplication;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.*;
import java.io.*;

import javax.swing.*;

import lowLevel.eval.Eval;
import lowLevel.producer.JavaCompilationError;
import mainProgram.Main;

import org.antlr.runtime.RecognitionException;

import primitiveOperation.*;
import strategy.MakeOneStep;
import strategy.MakeOneStep.StepName;
import structure.integrity.IntegrityChecker;
import structure.program.ClassTable;
import structure.resource.*;
import tools.FileManager;
import tools.DataStructures.WrapperException;
import visitors.VisitorTranslateZero;

import compiler.Mfj;
import compiler.Mfjc;

public class MainGui extends JFrame {
  
  // private HighlightedDocument document = new HighlightedDocument();
  private JTabbedPane jTabbedPane;
  private LHText text = new LHText();
  JScrollPane scrollPane = new JScrollPane(text);
  final JFileChooser fc = new JFileChooser();
  //ByteArrayOutputStream error = new ByteArrayOutputStream();
  ByteArrayOutputStream out = new ByteArrayOutputStream();
  PrintStream oldOut = System.out;
  PrintStream oldErr = System.err;
  {
    //text.setBorder(new LineNumberedBorder(LineNumberedBorder.LEFT_SIDE, LineNumberedBorder.LEFT_JUSTIFY, text));
    System.setErr(new PrintStream(out));//error
    System.setOut(new PrintStream(out));
  }
  //ClassTable ct = null;
  public void execute(ClassTable ct) {
    if (ct == null || ct.values(GreenResource.class).size() != ct.values().size()) {
      notifyErrorCantExecute();
      return;
    }
    try {
      System.out.println("\n execution of \"main\" ");
      Mfj.invokeMain();
    } catch (Throwable e) {
      e.printStackTrace();
      //displayOutputStatus("Print");
      //displayErrorStatus("Exception");
    }
    
    System.out.println("\n program terminated");
    displayOutputStatus("Main");
  }
  private void notifyErrorCantExecute() {
  //JOptionPane.showMessageDialog(null, "The compilation of the program is not done yet.", "Impossible Execution", JOptionPane.ERROR_MESSAGE);
  }
  public ClassTable compile() {
    FileManager.deleteDir(new File(Main.defaultOut, Eval.packageName));
    VisitorTranslateZero.generationNumber += 1;
    oldErr.print(out.toString());
    out.reset();
    jTabbedPane.removeAll();
    jTabbedPane.add("source", scrollPane);
    try {
      ClassTable ct = Mfjc.parseString(text.getText());
      boolean res = compileParsed(ct);
      return (res) ? ct : null;
      
    } catch (RecognitionException e1) {
      IntegrityChecker.printHintOverError(e1);
      e1.printStackTrace();
      //displayErrorStatus("Parsing Error - compilation does not even start");
      displayOutputStatus("Parsing Error - compilation does not even start");
      return null;
    } catch (IOException e) {
      e.printStackTrace();
      //displayErrorStatus("System Error - compilation does not even start");
      displayOutputStatus("System Error - compilation does not even start");
      return null;
    }
    
  }
  private boolean compileParsed(ClassTable ct) {
    ArrowPrinter ap = new ArrowPrinter();
    ap.initProgramStatus(ct);
    int i = 0;
    try {
      for (; ct.keySet(GreenResource.class).size() < ct.keySet().size(); i++) {
        String stepName = MakeOneStep.run(ct).repr();
        ap.displayProgramStatus(ct, "step" + i, "Last step was:  " + stepName);
      }
      Mfjc.generateMain(ct);
      return true;
    } catch (CircularityError e) {
      e.printStackTrace();
      ap.displayProgramStatus(ct, "step" + i, StepName.RedToYellowError.repr());
      return false;
    } catch (MetaChecRedToYellowError e) {
      e.printStackTrace();
      ap.displayProgramStatus(ct, "step" + i, StepName.RedToYellowError.repr());
      return false;
    } catch (WrapperException e) {
      ((Throwable) e.wrappedElement).printStackTrace();
      ap.displayProgramStatus(ct, "step" + i, StepName.NormalizeYellowError.repr());
      return false;
    } catch (YellowToGreenError e) {
      e.printStackTrace();
      ap.displayProgramStatus(ct, "step" + i, StepName.YellowToGreenError.repr());
      return false;
    } catch (JavaCompilationError e) {
      e.printStackTrace();
      displayOutputStatus("Compilation error in main expression");
      return false;
    } catch (Throwable e) {
      e.printStackTrace();
      displayOutputStatus("Internal Error");
      return false;
    }
  }
  /*
  private void displayErrorStatus(String round) {
    MonoView m = new MonoView();
    m.print.setText(error.toString());
    m.why.setText("Errors:");
    error.reset();
    jTabbedPane.add(round, m);
  }*/
  private void displayOutputStatus(String round) {
    MonoView m = new MonoView();
    m.print.setText(out.toString());
    m.why.setText("Console:");
    //out.reset();
    jTabbedPane.add(round, m);
  }
  private class ArrowPrinter {
    StringBuffer oldRed = new StringBuffer();
    StringBuffer oldYellow = new StringBuffer();
    StringBuffer oldGreen = new StringBuffer();
    String oldConsole = new String();
    private void initProgramStatus(ClassTable ct) {
      fillColors(ct, oldRed, oldYellow, oldGreen);
    }
    private void fillColors(ClassTable ct, StringBuffer red, StringBuffer yellow, StringBuffer green) {
      for (Resource r : ct.values(Resource.class))
        red.append(r);
      red.append(" \n ");
      for (YellowResource r : ct.values(YellowResource.class))
        yellow.append(r);
      yellow.append(" \n ");
      for (GreenResource r : ct.values(GreenResource.class))
        green.append(r);
      green.append(" \n ");
    }
    private void displayProgramStatus(ClassTable ct, String round, String stepName) {
      StepView sv = new StepView();
      sv.why.setText(stepName);
      StringBuffer red = new StringBuffer();
      StringBuffer yellow = new StringBuffer();
      StringBuffer green = new StringBuffer();
      fillColors(ct, red, yellow, green);
      sv.red1.setText(oldRed.toString());
      sv.red2.setText(red.toString());
      sv.yellow1.setText(oldYellow.toString());
      sv.yellow2.setText(yellow.toString());
      sv.green1.setText(oldGreen.toString());
      sv.green2.setText(green.toString());
      sv.console1.setText(oldConsole.toString());
      sv.console2.setText(out.toString());
      oldRed = red;
      oldYellow = yellow;
      oldGreen = green;
      oldConsole = out.toString();
      jTabbedPane.add(round, sv);
      //jTabbedPane.revalidate();
      sv.adjustSizes();
      
    }
  }
  public MainGui() {
    super("Meta Featherweight Jigsaw");
    Mfjc.setDir(new File(Main.defaultOut), null);
    Eval.setDir(new File(Main.defaultOut), Mfjc.getLoader());
    FileManager.createNew(Mfjc.getDir());
    FileManager.deleteDir(new File(Mfjc.getDir(), Eval.packageName));
    FileManager.deleteDir(new File(Mfjc.getDir(), "" + Mfjc.programCode));
    setLocation(50, 50);
    //document.setHighlightStyle(MFJLexer.class);
    scrollPane.setPreferredSize(new Dimension(820, 560));
    scrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    jTabbedPane = new JTabbedPane();
    jTabbedPane.add("source", scrollPane);
    JPanel contentPane = new JPanel(new BorderLayout());
    JPanel panel = new JPanel();
    panel.add(new JButton(new AbstractAction("loadFile") {
      public void actionPerformed(ActionEvent e) {
        int returnVal = fc.showOpenDialog(MainGui.this);
        if (returnVal == JFileChooser.APPROVE_OPTION) {
          File file = fc.getSelectedFile();
          text.setText(FileManager.loadFile(file));
          jTabbedPane.removeAll();
          jTabbedPane.add("source", scrollPane);
        }
      }
    }));
    panel.add(new JButton(new AbstractAction("compile and execute") {
      public void actionPerformed(ActionEvent e) {
        execute(compile());
      }
    }));
    //panel.add(new JButton(new AbstractAction("execute") {
    //  public void actionPerformed(ActionEvent e) {
    //    execute();
    //  }
    //}));
    contentPane.add(panel, BorderLayout.NORTH);
    contentPane.add(jTabbedPane, BorderLayout.CENTER);
    setContentPane(contentPane);
    addWindowListener(new WindowAdapter() {
      @Override
      public void windowClosing(WindowEvent e) {
        System.exit(0);
      }
      @Override
      public void windowActivated(WindowEvent e) {
        text.requestFocus();
      }
    });
    
    initDocument();
    pack();
    setVisible(true);
    //System.setProperty("java.class.path", defaultOut + File.pathSeparator + System.getProperty("java.class.path"));
  }
  private void initDocument() {
    String initString = //
    "" + //
        "class Foo={\n" + //
        "  class someMethodName(boolean b){\n" + //
        "    print(\"compile time execution\\n\");\n" + //
        "    if(b)return \n" + //
        "      {\n" + //
        "        int foo(){return 1;}\n" + //
        "      };\n" + //
        "    else return\n" + //
        "      {\n" + //
        "      int bar(){return 2;}\n" + //
        "      };\n" + //
        "    }\n" + //
        "  }\n" + //
        "class Bar=new Foo().someMethodName(true)\n" + //
        "main=print(new Bar().foo());\n" + //
        "//class Bar=new Foo().someMethodName(false)\n" + //
        "//main=print(new Bar().bar());\n";
    
    text.setText(initString);
  }
  /**
   * @param args ignored
   */
  public static void main(String[] args) {
    MainGui frame = new MainGui();
  }
}
