package lowLevel.eval;

import static tools.DataStructures.*;

import java.io.File;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.net.*;

import lowLevel.producer.JavaCompilationError;
import tools.Assertions;
import tools.FileManager;

/*
 * java expression --> java compiled/compile error
 * javaCompiled ->invoke
 * 
 * string [type (params ){body}]--> java compiled/compile error
 * 
 * javaCompiled ->invoke(Object...args)->Object
 * */

public class Eval {
  public static void setDir(File f, ClassLoader c) {
    FileManager.createNew(f);
    dir = f;
    if (c == null) try {
      c = new URLClassLoader(new URL[]{dir.toURI().toURL()}, ClassLoader.getSystemClassLoader());
    } catch (MalformedURLException e) {
      Assertions.userErrorAssert("", e);
    }
    ucl = c;
  }
  private static File dir;
  private static ClassLoader ucl;
  public static String packageName = "evalPredefinePackageName";
  private static int count = 0;
  public static CompiledExpression compile(String source) throws JavaCompilationError {
    StringBuffer sourceB = new StringBuffer(source);
    int insertHere = sourceB.indexOf("(");
    sourceB.insert(insertHere, "eval");
    String className = "UnnamedClass" + (count++);
    StringBuffer pre = new StringBuffer("public class " + className + "{\n public static ");
    pre.append(sourceB);
    pre.append("\n}");
    lowLevel.producer.JavaProducer.translate(dir, AL(T(//
        new StringBuffer(packageName),//
        className,//
        pre//
        )));
    String completeName = packageName + "." + className;
    try {
      //assert ucl == Mfjc.getLoader();
      Class<?> c = FileManager.loadClassDiskWait(completeName, ucl);
      Method[] ms = c.getDeclaredMethods();
      assert ms.length == 1;
      assert ms[0].getName().equals("eval");
      return new CompiledExpression(ms[0]);
    } catch (ClassNotFoundException e) {
      throw Assertions.reflectionError("", e);
    }
  }
  public static class CompiledExpression {
    Method m;
    CompiledExpression(Method m) {
      this.m = m;
    }
    public Object invoke(Object... args) throws IllegalArgumentException, IllegalAccessException, InvocationTargetException {
      return m.invoke(null, args);
    }
  }
  
}
