package lowLevel.producer;

import static tools.DataStructures.*;

import java.io.File;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.Collection;

import javax.tools.*;

import tools.Assertions;
import tools.DataStructures.Tuple3;

public class JavaProducer {
  
  /**
   * 
   * @param dir
   * @param list of packageName, classSimpleName, classString (no package);
   */
  public static void translate(File dir, ArrayList<Tuple3<StringBuffer, String, StringBuffer>> list) throws JavaCompilationError {
    if (list.isEmpty()) return;
    try {
      translateListAux(dir, list);
    } catch (URISyntaxException e) {
      throw Assertions.userErrorAssert("Invalid name", e);
    }
  }
  
  public static final JavaCompiler compiler = ToolProvider.getSystemJavaCompiler();
  private static DiagnosticCollector<JavaFileObject> diagnostics = new DiagnosticCollector<JavaFileObject>();
  
  private static void translateListAux(File dir, ArrayList<Tuple3<StringBuffer, String, StringBuffer>> list) throws URISyntaxException,
      JavaCompilationError {
    ArrayList<RamJavaFile> sources = new ArrayList<RamJavaFile>();
    processData(list, sources);
    compileClasses(dir, sources);
  }
  
  private static void processData(ArrayList<Tuple3<StringBuffer, String, StringBuffer>> list, ArrayList<RamJavaFile> sources)
      throws URISyntaxException {
    for (Tuple3<StringBuffer, String, StringBuffer> data : list) {
      StringBuffer packageName = data._1;
      String className = data._2;
      StringBuffer classString = data._3;
      StringBuffer classString2 = new StringBuffer();
      if (packageName.length() != 0) {
        classString2.append("package ");
        classString2.append(packageName);
        classString2.append(";\n");
      }
      classString2.append(classString);
      sources.add(new RamJavaFile(className, classString2.toString()));
    }
  }
  
  public static <T extends Collection<RamJavaFile>> void compileClasses(File dir, T sources) throws JavaCompilationError {
    assert compiler != null : "USE A JDK JAVA";
    PackageRespect jfm = new PackageRespect(dir, compiler.getStandardFileManager(diagnostics, null, null));
    JavaCompiler.CompilationTask task = compiler.getTask(
    //
        null, jfm, null, AL("-cp", dir.getAbsolutePath() + File.pathSeparator + System.getProperty("java.class.path")), null, sources);
    //BE CARE: no "\"" around the absolutePath, in fact iterable string already have
    //done this part of "parsing"
    boolean status = task.call();
    if (!status) throw new JavaCompilationError(//
        //"In " + expandSources(sources) + "\n-------------\nCompilation Error in translation to java.\n" + //
        diagnostics.getDiagnostics().toString());
  }
  private static <T extends Collection<RamJavaFile>> String expandSources(T sources) {
    StringBuffer result = new StringBuffer();
    for (RamJavaFile f : sources)
      result.append("\n-------------------\n" + f.programText);
    return result.toString();
  }
}
