package primitiveOperation;

import java.util.HashMap;
import java.util.HashSet;

import structure.astNode.Ast.ClassName;
import structure.program.ClassTable;
import structure.resource.Resource;
import structure.resource.YellowResource.MetaYellowResource;
import structure.resource.YellowResource.NormalizedYellowResource;
import visitors.VisitorCollectUsedTypesStar;
import visitors.VisitorCollectUsedTypesZero;

public class IsCircular {
  public static ClassName exampleOfCircularity(ClassTable ct) {
    assert ct.keySet(MetaYellowResource.class).isEmpty();
    HashMap<ClassName, HashSet<ClassName>> red = new HashMap<ClassName, HashSet<ClassName>>();
    HashMap<ClassName, HashSet<ClassName>> yellow = new HashMap<ClassName, HashSet<ClassName>>();
    for (Resource r : ct.values(Resource.class))
      red.put(r.name, new VisitorCollectUsedTypesZero().run(r.expression));
    
    for (NormalizedYellowResource r : ct.values(NormalizedYellowResource.class))
      yellow.put(r.name, new VisitorCollectUsedTypesStar().run(r.expression));
    //reflexivity
    for (ClassName n : yellow.keySet())
      yellow.get(n).add(n);
    //transitivity
    for (ClassName n : yellow.keySet()) {
      HashSet<ClassName> actual = yellow.get(n);
      while (true) {
        int size = actual.size();
        HashSet<ClassName> newActual = new HashSet<ClassName>();
        for (ClassName nn : actual)
          if (yellow.containsKey(nn)) newActual.addAll(yellow.get(nn));
        actual.addAll(newActual);
        if (size == actual.size()) break;
      }
    }
    HashMap<ClassName, HashSet<ClassName>> depend = new HashMap<ClassName, HashSet<ClassName>>();
    //dependDirect
    for (ClassName n : red.keySet()) {
      HashSet<ClassName> all = new HashSet<ClassName>();
      for (ClassName nn : red.get(n)) {
        all.add(nn);
        if (yellow.containsKey(nn)) all.addAll(yellow.get(nn));
      }
      all.retainAll(red.keySet());
      depend.put(n, all);
    }
    //depend transitivity
    for (ClassName n : depend.keySet()) {
      HashSet<ClassName> actual = depend.get(n);
      while (true) {
        int size = actual.size();
        HashSet<ClassName> newActual = new HashSet<ClassName>();
        for (ClassName nn : actual)
          if (depend.containsKey(nn)) newActual.addAll(depend.get(nn));
        actual.addAll(newActual);
        if (size == actual.size()) break;
      }
    }
    for (ClassName n : depend.keySet())
      if (depend.get(n).contains(n)) return n;
    return null;
  }
}
