package primitiveType;

import java.util.*;

import structure.astNode.Members;
import structure.astNode.Sequence;
import structure.astNode.Ast.*;
import structure.integrity.IntegrityChecker;
import structure.resource.GreenResource;
import visitors.*;

public class MFJClass {
  private final BaseClassValue value;
  public MFJClass(BaseClassValue value) {
    this.value = value;
  }
  @Override
  public String toString() {
    return value.toString();
  }
  @Override
  public boolean equals(Object other) {
    if (!other.getClass().equals(this.getClass())) return false;
    return value.equals(((MFJClass) other).value);
  }
  @Override
  public int hashCode() {
    return value.hashCode();
  }
  public BaseClassValue __extractCLZero() {
    return value;
  }
  public MFJName[] __operatorMembers() {
    Set<MemberName> ks = value.members.keySet();
    MFJName[] result = new MFJName[ks.size()];
    int i = 0;
    for (MemberName name : ks)
      result[i++] = new MFJName(name.value);
    return result;
  }
  public MFJClass __operatorSum(MFJClass other) {
    BaseClassValue bc = other.value;
    HashSet<ClassName> impl = new HashSet<ClassName>(value.impl);
    impl.addAll(bc.impl);
    Members m = new Members();
    fillConcreteMembers(value, m);
    fillConcreteMembers(bc, m);
    fillAbstractMembers(value, m);
    fillAbstractMembers(bc, m);
    if (!value.constructor.params.types.equals(bc.constructor.params.types)) //
      throw new PredefinedExceptions.SumException("Constructor params are not equals");
    
    Constructor c = composeConstructors(value.constructor, bc.constructor);
    return new MFJClass(new BaseClassValue(impl, m, c));
  }
  private static Constructor composeConstructors(Constructor c1, Constructor c2) {
    //rename c2 params as _0 _1 _2 ...
    for (int i = 0; i < c1.params.names.size(); i++) {
      VariableName nVn = new VariableName(IntegrityChecker.forbiddenPrefix + i);
      c2 = new VisitorRenameFromToVariableZero(c2.params.names.get(i), nVn).visit(c2);
    }
    //rename c2 params as c1 ones
    for (int i = 0; i < c1.params.names.size(); i++) {
      c2 = new VisitorRenameFromToVariableZero(c2.params.names.get(i), c1.params.names.get(i)).visit(c2);
    }
    ArrayList<MemberName> names = new ArrayList<MemberName>(c1.names);
    ArrayList<Expression> expression = new ArrayList<Expression>(c1.expressions);
    names.addAll(c2.names);
    expression.addAll(c2.expressions);
    
    Constructor c = new Constructor(c1.params, names, expression);
    return c;
  }
  private void fillAbstractMembers(BaseClassValue bc, Members m) {
    for (AbstractField f : bc.members.values(AbstractField.class))
      if (m.isContained(Member.class, f.name)) {
        if (compatible(f, m.get(Member.class, f.name))) continue;
        throw new PredefinedExceptions.SumException("Field " + f.name + " type is not equal");
      } else m.put(f.name, f);
    
    for (AbstractMethod f : bc.members.values(AbstractMethod.class))
      if (m.isContained(Member.class, f.name)) {
        if (compatible(f, m.get(Member.class, f.name))) continue;
        throw new PredefinedExceptions.SumException("Method " + f.name + " type is not equal");
      }
  }
  private void fillConcreteMembers(BaseClassValue bc, Members m) {
    for (Field f : bc.members.values(Field.class))
      if (m.isContained(Member.class, f.name)) throw new PredefinedExceptions.SumException("Field " + f.name + " conflicting definition");
      else m.put(f.name, f);
    for (Method f : bc.members.values(Method.class))
      if (m.isContained(Member.class, f.name)) throw new PredefinedExceptions.SumException("Method " + f.name + " conflicting definition");
      else m.put(f.name, f);
  }
  
  private boolean compatible(Member f, Member ff) {
    VisitorMemberMakeAbstractNormalizeParameters v = new VisitorMemberMakeAbstractNormalizeParameters();
    VisitorMemberAlias vAl = new VisitorMemberAlias(new MemberName("_"));
    return f.accept(v).accept(vAl).equals(ff.accept(v).accept(vAl));
    
  }
  public MFJClass __operatorRestrict(MFJName nname) {
    MemberName name = nname.__extract();
    Members ms = new Members();
    if (!value.members.isContained(IDefined.class, name)) throw new PredefinedExceptions.RestrictException("Member not defined");
    for (Member m : value.members.values()) {
      if (!m.getName().equals(name)) ms.put(m.getName(), m);
      else ms.put(m.getName(), m.accept(new VisitorMemberMakeAbstract()));
      
    }
    Constructor c = value.constructor;
    if (value.members.isContained(Field.class, name)) {
      ArrayList<MemberName> mn = new ArrayList<MemberName>();
      ArrayList<Expression> exp = new ArrayList<Expression>();
      for (int i = 0; i < c.names.size(); i++) {
        if (c.names.get(i).equals(name)) continue;
        mn.add(c.names.get(i));
        exp.add(c.expressions.get(i));
      }
      c = new Constructor(c.params, mn, exp);
    }
    return new MFJClass(new BaseClassValue(value.impl, ms, c));
  }
  public MFJClass __operatorAlias(MFJName nname, MFJName nname2) {
    MemberName name = nname.__extract();
    MemberName name2 = nname2.__extract();
    Members ms = Sequence.mMember(value.members.values().toArray(new Member[0]));
    if (!value.members.isContained(IDefined.class, name)) throw new PredefinedExceptions.AliasException("Member not defined");
    if (value.members.isContained(IDefined.class, name2)) throw new PredefinedExceptions.AliasException("Member already defined");
    Member selected = value.members.get(Member.class, name);
    if (value.members.isContained(IAbstract.class, name2)) {
      if (!compatible(//
          value.members.get(IAbstract.class, name2),//
          selected)) throw new PredefinedExceptions.AliasException("Type is not equal");
      ms.remove(Member.class, name2);
    }
    ms.put(name2, selected.accept(new VisitorMemberAlias(name2)));
    Constructor c = value.constructor;
    if (selected instanceof Field) {
      ArrayList<MemberName> mn = new ArrayList<MemberName>();
      ArrayList<Expression> exp = new ArrayList<Expression>();
      for (int i = 0; i < c.names.size(); i++) {
        if (c.names.get(i).equals(name)) {
          mn.add(name2);
          exp.add(c.expressions.get(i));
        }
        mn.add(c.names.get(i));
        exp.add(c.expressions.get(i));
      }
      c = new Constructor(c.params, mn, exp);
    }
    return new MFJClass(new BaseClassValue(value.impl, ms, c));
  }
  
  public MFJClass __operatorRedirect(MFJName nname2, MFJName nname) {
    MemberName destinationName = nname.__extract();
    MemberName sourceName = nname2.__extract();
    Members ms = Sequence.mMember(value.members.values().toArray(new Member[0]));
    if (!value.members.isContained(IAbstract.class, sourceName)) throw new PredefinedExceptions.RedirectException("Member is not abstract");
    if (value.members.isContained(Member.class, destinationName)) {
      Member selected = value.members.get(Member.class, destinationName);
      if (!compatible(//
          value.members.get(Member.class, sourceName),//
          selected)) throw new PredefinedExceptions.RedirectException("Type is not equal");
    }
    Members newM = new Members();
    for (IField f : value.members.values(IField.class))
      newM.put(f.getName(), f);
    for (AbstractMethod f : value.members.values(AbstractMethod.class))
      newM.put(f.getName(), f);
    for (Method f : value.members.values(Method.class))
      newM.put(f.getName(), new Method(f.returnType, f.name, f.params, (Block) VisitorRenameMemberNameZero.run(f.body, sourceName, destinationName)));
    if (!isNeeded(value.impl, destinationName)) newM.remove(IAbstract.class, sourceName);
    if (!value.members.isContained(Member.class, destinationName))
      newM.put(destinationName, value.members.get(IAbstract.class, sourceName).accept(new VisitorMemberAlias(destinationName)));
    return new MFJClass(new BaseClassValue(value.impl, newM, value.constructor));
  }
  private boolean isNeeded(HashSet<ClassName> impl, MemberName name) {
    for (ClassName cn : impl) {
      BaseClassValue bc = new GreenResource(cn).getValue();
      if (bc.members.isContained(Member.class, name)) return true;
    }//one level is enough, no "auto" abstract injection.
    return false;
  }
  
}