package structure.astNode;

import java.util.ArrayList;
import java.util.HashSet;

import visitors.Visitor;
import visitors.VisitorToString;

public interface Ast {
  <T> T accept(Visitor<T> v);
  public static interface Expression extends Ast {}
  public static interface Value extends Expression {}
  public static interface Statement extends Ast {}
  public static interface Member extends Ast {
    MemberName getName();
  }
  public static interface IField extends Member {}
  public static interface IMethod extends Member {}
  public static interface IAbstract extends Member {}
  public static interface IDefined extends Member {}
  public static interface ValidStmExp extends Ast {}
  static abstract class AstImpl implements Ast {
    @Override
    public String toString() {
      return new VisitorToString().run(this).toString();
    }
  }
  public static class IntValue extends AstImpl implements Value {
    public IntValue(int value) {
      super();
      this.value = value;
    }
    public final int value;
    public <T> T accept(Visitor<T> v) {
      return v.visit(this);
    }
    @Override
    public boolean equals(Object other) {
      return EqualAst.areEquals(this, other);
    }
    @Override
    public int hashCode() {
      return HashAst.hash(this);
    }
  }
  public abstract static class IdHolder extends AstImpl {
    public IdHolder(String value) {
      super();
      this.value = value;
    }
    public final String value;
  }
  public static class StringValue extends AstImpl implements Value {
    public StringValue(String value) {
      super();
      this.value = value;
    }
    public final String value;
    @Override
    public <T> T accept(Visitor<T> v) {
      return v.visit(this);
    }
    @Override
    public boolean equals(Object other) {
      return EqualAst.areEquals(this, other);
    }
    @Override
    public int hashCode() {
      return HashAst.hash(this);
    }
    
  }
  public static class BoolValue extends AstImpl implements Value {
    public BoolValue(boolean value) {
      super();
      this.value = value;
    }
    public final boolean value;
    @Override
    public <T> T accept(Visitor<T> v) {
      return v.visit(this);
    }
    @Override
    public boolean equals(Object other) {
      return EqualAst.areEquals(this, other);
    }
    @Override
    public int hashCode() {
      return HashAst.hash(this);
    }
  }
  public static class ClassNameValue extends IdHolder implements Value {
    public ClassNameValue(String value) {
      super(value);
    }
    @Override
    public <T> T accept(Visitor<T> v) {
      return v.visit(this);
    }
    @Override
    public boolean equals(Object other) {
      return EqualAst.areEquals(this, other);
    }
    @Override
    public int hashCode() {
      return HashAst.hash(this);
    }
    
  }
  public static class MemberNameValue extends IdHolder implements Value {
    public MemberNameValue(String value) {
      super(value);
    }
    @Override
    public <T> T accept(Visitor<T> v) {
      return v.visit(this);
    }
    @Override
    public boolean equals(Object other) {
      return EqualAst.areEquals(this, other);
    }
    @Override
    public int hashCode() {
      return HashAst.hash(this);
    }
    
  }
  
  public static class ClassName extends IdHolder implements Ast {
    public ClassName(String value) {
      super(value);
    }
    @Override
    public <T> T accept(Visitor<T> v) {
      return v.visit(this);
    }
    @Override
    public boolean equals(Object other) {
      return EqualAst.areEquals(this, other);
    }
    @Override
    public int hashCode() {
      return HashAst.hash(this);
    }
    
  }
  public static class MemberName extends IdHolder implements Ast {
    public MemberName(String value) {
      super(value);
    }
    @Override
    public <T> T accept(Visitor<T> v) {
      return v.visit(this);
    }
    @Override
    public boolean equals(Object other) {
      return EqualAst.areEquals(this, other);
    }
    @Override
    public int hashCode() {
      return HashAst.hash(this);
    }
    
  }
  
  public static class VariableName extends IdHolder implements Ast {
    public VariableName(String value) {
      super(value);
    }
    @Override
    public <T> T accept(Visitor<T> v) {
      return v.visit(this);
    }
    @Override
    public boolean equals(Object other) {
      return EqualAst.areEquals(this, other);
    }
    @Override
    public int hashCode() {
      return HashAst.hash(this);
    }
    
  }
  public static class VariableAccess extends AstImpl implements Expression {
    public VariableAccess(VariableName value) {
      super();
      this.value = value;
    }
    public final VariableName value;
    @Override
    public <T> T accept(Visitor<T> v) {
      return v.visit(this);
    }
    @Override
    public boolean equals(Object other) {
      return EqualAst.areEquals(this, other);
    }
    @Override
    public int hashCode() {
      return HashAst.hash(this);
    }
  }
  public static class InternalFieldAccess extends AstImpl implements Expression {
    public InternalFieldAccess(MemberName value) {
      super();
      this.value = value;
    }
    public final MemberName value;
    @Override
    public <T> T accept(Visitor<T> v) {
      return v.visit(this);
    }
    @Override
    public boolean equals(Object other) {
      return EqualAst.areEquals(this, other);
    }
    @Override
    public int hashCode() {
      return HashAst.hash(this);
    }
  }
  public static class InternalMethodCall extends AstImpl implements Expression, ValidStmExp {
    public InternalMethodCall(MemberName value, ParamsUse params) {
      super();
      this.value = value;
      this.params = params;
    }
    public final MemberName value;
    public final ParamsUse params;
    @Override
    public <T> T accept(Visitor<T> v) {
      return v.visit(this);
    }
    @Override
    public boolean equals(Object other) {
      return EqualAst.areEquals(this, other);
    }
    @Override
    public int hashCode() {
      return HashAst.hash(this);
    }
  }
  public static class ClientFieldAccess extends AstImpl implements Expression {
    public ClientFieldAccess(Expression reciver, MemberName value) {
      super();
      this.value = value;
      this.reciver = reciver;
    }
    public final MemberName value;
    public final Expression reciver;
    @Override
    public <T> T accept(Visitor<T> v) {
      return v.visit(this);
    }
    @Override
    public boolean equals(Object other) {
      return EqualAst.areEquals(this, other);
    }
    @Override
    public int hashCode() {
      return HashAst.hash(this);
    }
  }
  public static class ClientMethodCall extends AstImpl implements Expression, ValidStmExp {
    public ClientMethodCall(Expression reciver, MemberName value, ParamsUse params) {
      super();
      this.value = value;
      this.params = params;
      this.reciver = reciver;
    }
    public final MemberName value;
    public final ParamsUse params;
    public final Expression reciver;
    @Override
    public <T> T accept(Visitor<T> v) {
      return v.visit(this);
    }
    @Override
    public boolean equals(Object other) {
      return EqualAst.areEquals(this, other);
    }
    @Override
    public int hashCode() {
      return HashAst.hash(this);
    }
  }
  public static class BaseClassValue extends AstImpl implements Value {
    
    public BaseClassValue(HashSet<ClassName> impl, Members members, Constructor constructor) {
      super();
      this.impl = impl;
      this.members = members;
      this.constructor = constructor;
    }
    public final HashSet<ClassName> impl;
    public final Members members;
    public final Constructor constructor;
    @Override
    public <T> T accept(Visitor<T> v) {
      return v.visit(this);
    }
    @Override
    public boolean equals(Object other) {
      return EqualAst.areEquals(this, other);
    }
    @Override
    public int hashCode() {
      return HashAst.hash(this);
    }
  }
  public static class ParamsDec extends AstImpl implements Ast {
    public ParamsDec(ArrayList<ClassName> types, ArrayList<VariableName> names) {
      super();
      //      IntegrityChecker.checkDuplication(names);//in parsing
      this.types = types;
      this.names = names;
    }
    public final ArrayList<ClassName> types;
    public final ArrayList<VariableName> names;
    @Override
    public <T> T accept(Visitor<T> v) {
      return v.visit(this);
    }
    @Override
    public boolean equals(Object other) {
      return EqualAst.areEquals(this, other);
    }
    @Override
    public int hashCode() {
      return HashAst.hash(this);
    }
  }
  public static class ParamsUse extends AstImpl implements Ast {
    public ParamsUse(ArrayList<Expression> actual) {
      super();
      this.actual = actual;
    }
    public ArrayList<Expression> actual;
    @Override
    public <T> T accept(Visitor<T> v) {
      return v.visit(this);
    }
    @Override
    public boolean equals(Object other) {
      return EqualAst.areEquals(this, other);
    }
    @Override
    public int hashCode() {
      return HashAst.hash(this);
    }
  }
  public static class Constructor extends AstImpl implements Ast {
    public Constructor(ParamsDec params, ArrayList<MemberName> names, ArrayList<Expression> expressions) {
      super();
      this.params = params;
      this.names = names;
      this.expressions = expressions;
    }
    public final ParamsDec params;
    public final ArrayList<MemberName> names;
    public final ArrayList<Expression> expressions;
    @Override
    public <T> T accept(Visitor<T> v) {
      return v.visit(this);
    }
    @Override
    public boolean equals(Object other) {
      return EqualAst.areEquals(this, other);
    }
    @Override
    public int hashCode() {
      return HashAst.hash(this);
    }
  }
  public static class Field extends AstImpl implements Member, IField, IDefined {
    public Field(ClassName type, MemberName name) {
      super();
      this.type = type;
      this.name = name;
    }
    public final ClassName type;
    public final MemberName name;
    @Override
    public MemberName getName() {
      return this.name;
    }
    @Override
    public <T> T accept(Visitor<T> v) {
      return v.visit(this);
    }
    @Override
    public boolean equals(Object other) {
      return EqualAst.areEquals(this, other);
    }
    @Override
    public int hashCode() {
      return HashAst.hash(this);
    }
  }
  public static class AbstractField extends AstImpl implements Member, IField, IAbstract {
    public AbstractField(ClassName type, MemberName name) {
      super();
      this.type = type;
      this.name = name;
    }
    public final ClassName type;
    public final MemberName name;
    @Override
    public MemberName getName() {
      return this.name;
    }
    @Override
    public <T> T accept(Visitor<T> v) {
      return v.visit(this);
    }
    @Override
    public boolean equals(Object other) {
      return EqualAst.areEquals(this, other);
    }
    @Override
    public int hashCode() {
      return HashAst.hash(this);
    }
  }
  public static class Method extends AstImpl implements Member, IMethod, IDefined {
    public Method(ClassName returnType, MemberName name, ParamsDec params, Block body) {
      super();
      this.returnType = returnType;
      this.name = name;
      this.params = params;
      this.body = body;
      
    }
    public final ClassName returnType;
    public final MemberName name;
    public final ParamsDec params;
    public final Block body;
    @Override
    public MemberName getName() {
      return this.name;
    }
    @Override
    public <T> T accept(Visitor<T> v) {
      return v.visit(this);
    }
    @Override
    public boolean equals(Object other) {
      return EqualAst.areEquals(this, other);
    }
    @Override
    public int hashCode() {
      return HashAst.hash(this);
    }
  }
  public static class AbstractMethod extends AstImpl implements Member, IMethod, IAbstract {
    public AbstractMethod(ClassName returnType, MemberName name, ParamsDec params) {
      super();
      this.returnType = returnType;
      this.name = name;
      this.params = params;
    }
    public final ClassName returnType;
    public final MemberName name;
    public final ParamsDec params;
    @Override
    public MemberName getName() {
      return this.name;
    }
    @Override
    public <T> T accept(Visitor<T> v) {
      return v.visit(this);
    }
    @Override
    public boolean equals(Object other) {
      return EqualAst.areEquals(this, other);
    }
    @Override
    public int hashCode() {
      return HashAst.hash(this);
    }
  }
  public static class Block extends AstImpl implements Statement {
    public Block(ArrayList<Statement> statements) {
      super();
      this.statements = statements;
    }
    public final ArrayList<Statement> statements;
    public <T> T accept(Visitor<T> v) {
      return v.visit(this);
    }
    @Override
    public boolean equals(Object other) {
      return EqualAst.areEquals(this, other);
    }
    @Override
    public int hashCode() {
      return HashAst.hash(this);
    }
  }
  public static class Foreach extends AstImpl implements Statement {
    
    public final ClassName type;
    public final VariableName name;
    public final Expression expression;
    public final Statement body;
    public Foreach(ClassName type, VariableName name, Expression expression, Statement body) {
      super();
      this.type = type;
      this.name = name;
      this.expression = expression;
      this.body = body;
    }
    
    public <T> T accept(Visitor<T> v) {
      return v.visit(this);
    }
    @Override
    public boolean equals(Object other) {
      return EqualAst.areEquals(this, other);
    }
    @Override
    public int hashCode() {
      return HashAst.hash(this);
    }
  }
  public static class Try extends AstImpl implements Statement {
    public final ArrayList<VariableName> names;
    public final ArrayList<ClassName> types;
    public final Block _try;
    public Try(Block _try, ArrayList<ClassName> types, ArrayList<VariableName> names, ArrayList<Block> blocks) {
      super();
      this._try = _try;
      this.types = types;
      this.names = names;
      this.blocks = blocks;
    }
    public final ArrayList<Block> blocks;
    public <T> T accept(Visitor<T> v) {
      return v.visit(this);
    }
    @Override
    public boolean equals(Object other) {
      return EqualAst.areEquals(this, other);
    }
    @Override
    public int hashCode() {
      return HashAst.hash(this);
    }
  }
  public static class If extends AstImpl implements Statement {
    public If(Expression condition, Statement then, Statement _else) {
      super();
      this.condition = condition;
      this.then = then;
      this._else = _else;
    }
    public final Expression condition;
    public final Statement then;
    public final Statement _else;
    @Override
    public <T> T accept(Visitor<T> v) {
      return v.visit(this);
    }
    @Override
    public boolean equals(Object other) {
      return EqualAst.areEquals(this, other);
    }
    @Override
    public int hashCode() {
      return HashAst.hash(this);
    }
  }
  public static class While extends AstImpl implements Statement {
    public While(Expression condition, Statement body) {
      super();
      this.condition = condition;
      this.body = body;
    }
    public final Expression condition;
    public final Statement body;
    @Override
    public <T> T accept(Visitor<T> v) {
      return v.visit(this);
    }
    @Override
    public boolean equals(Object other) {
      return EqualAst.areEquals(this, other);
    }
    @Override
    public int hashCode() {
      return HashAst.hash(this);
    }
  }
  public static class VariableDeclaration extends AstImpl implements Statement {
    public VariableDeclaration(ClassName type, VariableName name, Expression initialization) {
      super();
      this.type = type;
      this.name = name;
      this.initialization = initialization;
    }
    public final ClassName type;
    public final VariableName name;
    public final Expression initialization;
    @Override
    public <T> T accept(Visitor<T> v) {
      return v.visit(this);
    }
    @Override
    public boolean equals(Object other) {
      return EqualAst.areEquals(this, other);
    }
    @Override
    public int hashCode() {
      return HashAst.hash(this);
    }
  }
  public static class VariableAssignament extends AstImpl implements Statement {
    public VariableAssignament(VariableName name, Expression assignedValue) {
      super();
      this.name = name;
      this.assignedValue = assignedValue;
    }
    public final VariableName name;
    public final Expression assignedValue;
    @Override
    public <T> T accept(Visitor<T> v) {
      return v.visit(this);
    }
    @Override
    public boolean equals(Object other) {
      return EqualAst.areEquals(this, other);
    }
    @Override
    public int hashCode() {
      return HashAst.hash(this);
    }
  }
  
  public static class StmExp extends AstImpl implements Statement {
    public StmExp(Expression expression) {
      super();
      this.expression = expression;
    }
    public Expression expression;
    @Override
    public <T> T accept(Visitor<T> v) {
      return v.visit(this);
    }
    @Override
    public boolean equals(Object other) {
      return EqualAst.areEquals(this, other);
    }
    @Override
    public int hashCode() {
      return HashAst.hash(this);
    }
  }
  
  public static class Return extends AstImpl implements Statement {
    public Return(Expression expression) {
      super();
      this.expression = expression;
    }
    public Expression expression;
    @Override
    public <T> T accept(Visitor<T> v) {
      return v.visit(this);
    }
    @Override
    public boolean equals(Object other) {
      return EqualAst.areEquals(this, other);
    }
    @Override
    public int hashCode() {
      return HashAst.hash(this);
    }
  }
  public static class Throw extends AstImpl implements Statement {
    public Throw(Expression expression) {
      super();
      this.expression = expression;
    }
    public Expression expression;
    @Override
    public <T> T accept(Visitor<T> v) {
      return v.visit(this);
    }
    @Override
    public boolean equals(Object other) {
      return EqualAst.areEquals(this, other);
    }
    @Override
    public int hashCode() {
      return HashAst.hash(this);
    }
  }
  public static class New extends AstImpl implements Expression, ValidStmExp {
    public New(ClassName type, ParamsUse params) {
      super();
      this.type = type;
      this.params = params;
    }
    public final ClassName type;
    public final ParamsUse params;
    @Override
    public <T> T accept(Visitor<T> v) {
      return v.visit(this);
    }
    @Override
    public boolean equals(Object other) {
      return EqualAst.areEquals(this, other);
    }
    @Override
    public int hashCode() {
      return HashAst.hash(this);
    }
  }
  public static class NewArray extends AstImpl implements Expression {
    public NewArray(ClassName type, ParamsUse params) {
      super();
      this.type = type;
      this.params = params;
    }
    public final ClassName type;
    public final ParamsUse params;
    @Override
    public <T> T accept(Visitor<T> v) {
      return v.visit(this);
    }
    @Override
    public boolean equals(Object other) {
      return EqualAst.areEquals(this, other);
    }
    @Override
    public int hashCode() {
      return HashAst.hash(this);
    }
  }
  public static class Print extends AstImpl implements Statement {
    public Print(Expression expression) {
      super();
      this.expression = expression;
    }
    public final Expression expression;
    @Override
    public <T> T accept(Visitor<T> v) {
      return v.visit(this);
    }
    @Override
    public boolean equals(Object other) {
      return EqualAst.areEquals(this, other);
    }
    @Override
    public int hashCode() {
      return HashAst.hash(this);
    }
  }
  public static class BinOpExp extends AstImpl implements Expression {
    public BinOpExp(Expression left, BinOp op, Expression right) {
      super();
      this.op = op;
      this.left = left;
      this.right = right;
    }
    public final BinOp op;
    public final Expression left;
    public final Expression right;
    @Override
    public <T> T accept(Visitor<T> v) {
      return v.visit(this);
    }
    @Override
    public boolean equals(Object other) {
      return EqualAst.areEquals(this, other);
    }
    @Override
    public int hashCode() {
      return HashAst.hash(this);
    }
  }
  public static class UnOpExp extends AstImpl implements Expression {
    public UnOpExp(UnOp op, Expression inner) {
      super();
      this.op = op;
      this.inner = inner;
    }
    public final UnOp op;
    public final Expression inner;
    @Override
    public <T> T accept(Visitor<T> v) {
      return v.visit(this);
    }
    @Override
    public boolean equals(Object other) {
      return EqualAst.areEquals(this, other);
    }
    @Override
    public int hashCode() {
      return HashAst.hash(this);
    }
  }
  
  public static class AccessArray extends AstImpl implements Expression {
    public AccessArray(Expression targhet, Expression inner) {
      super();
      this.targhet = targhet;
      this.inner = inner;
    }
    public final Expression targhet;
    public final Expression inner;
    @Override
    public <T> T accept(Visitor<T> v) {
      return v.visit(this);
    }
    @Override
    public boolean equals(Object other) {
      return EqualAst.areEquals(this, other);
    }
    @Override
    public int hashCode() {
      return HashAst.hash(this);
    }
  }
  
  public static class GetMembers extends AstImpl implements Expression {
    public GetMembers(Expression targhet) {
      super();
      this.targhet = targhet;
    }
    public final Expression targhet;
    @Override
    public <T> T accept(Visitor<T> v) {
      return v.visit(this);
    }
    @Override
    public boolean equals(Object other) {
      return EqualAst.areEquals(this, other);
    }
    @Override
    public int hashCode() {
      return HashAst.hash(this);
    }
  }
  public static class RestrictExp extends AstImpl implements Expression {
    public RestrictExp(Expression targhet, Expression inner) {
      super();
      this.targhet = targhet;
      this.inner = inner;
    }
    public final Expression targhet;
    public final Expression inner;
    @Override
    public <T> T accept(Visitor<T> v) {
      return v.visit(this);
    }
    @Override
    public boolean equals(Object other) {
      return EqualAst.areEquals(this, other);
    }
    @Override
    public int hashCode() {
      return HashAst.hash(this);
    }
  }
  public static class AliasExp extends AstImpl implements Expression {
    public AliasExp(Expression targhet, Expression left, Expression right) {
      super();
      this.targhet = targhet;
      this.left = left;
      this.right = right;
    }
    public final Expression targhet;
    public final Expression left;
    public final Expression right;
    @Override
    public <T> T accept(Visitor<T> v) {
      return v.visit(this);
    }
    @Override
    public boolean equals(Object other) {
      return EqualAst.areEquals(this, other);
    }
    @Override
    public int hashCode() {
      return HashAst.hash(this);
    }
  }
  public static class RedirectExp extends AstImpl implements Expression {
    public RedirectExp(Expression targhet, Expression left, Expression right) {
      super();
      this.targhet = targhet;
      this.left = left;
      this.right = right;
    }
    public final Expression targhet;
    public final Expression left;
    public final Expression right;
    @Override
    public <T> T accept(Visitor<T> v) {
      return v.visit(this);
    }
    @Override
    public boolean equals(Object other) {
      return EqualAst.areEquals(this, other);
    }
    @Override
    public int hashCode() {
      return HashAst.hash(this);
    }
  }
  
}
