package structure.astNode;

import structure.astNode.Ast.*;
import structure.astNode.Ast.StringValue;

class HashAst {
  public static int hash(IntValue e) {
    return e.value;
  }
  public static int hash(StringValue e) {
    return e.value.hashCode();
  }
  public static int hash(BoolValue e) {
    return (e.value) ? 1 : 0;
  }
  public static int hash(ClassNameValue e) {
    return e.value.hashCode();
  }
  public static int hash(MemberNameValue e) {
    return e.value.hashCode();
  }
  public static int hash(ClassName e) {
    return e.value.hashCode();
  }
  public static int hash(MemberName e) {
    return e.value.hashCode();
  }
  public static int hash(VariableName e) {
    return e.value.hashCode();
  }
  public static int hash(VariableAccess e) {
    return e.value.hashCode();
  }
  public static int hash(InternalFieldAccess e) {
    return e.value.hashCode();
  }
  public static int hash(InternalMethodCall e) {
    return e.value.hashCode() //
        + e.params.hashCode() * 31;
  }
  public static int hash(ClientFieldAccess e) {
    return e.reciver.hashCode() //
        + e.value.hashCode() * 31;
  }
  public static int hash(ClientMethodCall e) {
    return e.reciver.hashCode() //
        + e.value.hashCode() * 15 //
        + e.params.hashCode() * 63;
  }
  public static int hash(BaseClassValue e) {
    return e.impl.hashCode()//
        + e.members.values().hashCode() * 15//
        + e.constructor.hashCode() * 63;
    
  }
  public static int hash(ParamsDec e) {
    return e.names.hashCode()//
        + e.types.hashCode() * 31;
  }
  public static int hash(ParamsUse e) {
    return e.actual.hashCode();
  }
  public static int hash(Constructor e) {
    return e.params.hashCode() //
        + e.names.hashCode() * 15 //
        + e.expressions.hashCode() * 63;
  }
  public static int hash(Field e) {
    return e.type.hashCode()//
        + e.name.hashCode() * 31;
  }
  public static int hash(AbstractField e) {
    return e.type.hashCode()//
        + e.name.hashCode() * 31;
  }
  public static int hash(Method e) {
    return e.returnType.hashCode() //
        + e.name.hashCode() * 15 //
        + e.params.hashCode() * 31 //
        + e.body.hashCode() * 63;
  }
  public static int hash(Block e) {
    return e.statements.hashCode();
  }
  public static int hash(If e) {
    return e.condition.hashCode()//
        + e.then.hashCode() * 15 //
        + e._else.hashCode() * 63;
  }
  public static int hash(While e) {
    return e.condition.hashCode() //
        + e.body.hashCode() * 31;
  }
  public static int hash(VariableDeclaration e) {
    return e.type.hashCode() //
        + e.name.hashCode() * 15 //
        + e.initialization.hashCode() * 65;
  }
  public static int hash(Return e) {
    return e.expression.hashCode();
  }
  public static int hash(AbstractMethod e) {
    return e.returnType.hashCode() //
        + e.name.hashCode() * 15 //
        + e.params.hashCode() * 63;
  }
  public static int hash(New e) {
    return e.type.hashCode()//
        + e.params.hashCode() * 31;
  }
  public static int hash(NewArray e) {
    return e.type.hashCode()//
        + e.params.hashCode() * 31;
  }
  public static int hash(Print e) {
    return e.expression.hashCode();
  }
  public static int hash(BinOpExp e) {
    return e.left.hashCode() //
        + e.op.hashCode() * 15 //
        + e.right.hashCode() * 63;
  }
  public static int hash(UnOpExp e) {
    return e.op.hashCode()//
        + e.inner.hashCode() * 31;
  }
  public static int hash(RestrictExp e) {
    return e.targhet.hashCode()//
        + e.inner.hashCode() * 31;
  }
  public static int hash(AliasExp e) {
    return e.targhet.hashCode()//
        + e.left.hashCode() * 15//
        + e.right.hashCode() * 63;
  }
  public static int hash(RedirectExp e) {
    return e.targhet.hashCode()//
        + e.left.hashCode() * 15//
        + e.right.hashCode() * 63;
  }
  public static int hash(StmExp e) {
    return e.expression.hashCode();
  }
  public static int hash(VariableAssignament e) {
    return e.name.hashCode()//
        + e.assignedValue.hashCode() * 31;
  }
  public static int hash(Throw e) {
    return e.expression.hashCode();
  }
  public static int hash(AccessArray e) {
    return e.targhet.hashCode()//
        + e.inner.hashCode() * 31;
  }
  public static int hash(GetMembers e) {
    return e.targhet.hashCode();
  }
  public static int hash(Try e) {
    return e._try.hashCode()//
        + e.types.hashCode() * 15//
        + e.names.hashCode() * 31//
        + e.blocks.hashCode() * 63;
  }
  public static int hash(Foreach e) {
    return e.type.hashCode()//
        + e.name.hashCode() * 15//
        + e.expression.hashCode() * 31//
        + e.body.hashCode() * 63;
  }
  
}
