package testUtils;

import static org.testng.AssertJUnit.*;

import java.io.File;
import java.io.StringReader;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;

import lowLevel.eval.Eval;
import lowLevel.eval.Eval.CompiledExpression;

import org.antlr.runtime.ANTLRReaderStream;
import org.antlr.runtime.CommonTokenStream;

import structure.antlrGenerated.MFJGrammarLexer;
import structure.antlrGenerated.MFJGrammarParser;
import structure.astNode.Ast;
import structure.program.ClassTable;
import structure.resource.Resource;
import tools.Assertions;
import tools.FileManager;
import visitors.VisitorToAstJavaCode;
import visitors.VisitorToString;

public class TestHelper {
  public static final File testTempDir = new File("testTempDir");
  public static ArrayList<File> collectTestFiles() {
    ArrayList<File> result = new ArrayList<File>();
    File outdir = new File("..\\..\\..\\pOut");
    if (!outdir.exists()) outdir.mkdir();
    assert outdir.isDirectory();
    
    ArrayList<File> files = FileManager.getAllFile(new File("pTests"));
    for (File f : files)
      if (f.isDirectory()) {
        if (!privateDirectory(f)) for (File f1 : FileManager.getAllFile(f))
          result.add(f1);
      } else result.add(f);
    return result;
  }
  private static boolean privateDirectory(File f) {
    return f.getName().charAt(0) == '.';
  }
  
  static public Object parseProduction(String prodName, String code) throws Throwable {
    ANTLRReaderStream input = new ANTLRReaderStream(new StringReader(code));
    MFJGrammarLexer lexer = new MFJGrammarLexer(input);
    CommonTokenStream tokens = new CommonTokenStream(lexer);
    MFJGrammarParser parser = new MFJGrammarParser(tokens);
    Method m = parser.getClass().getMethod(prodName);
    try {
      Object result = m.invoke(parser);
      //if (parser.eList.isEmpty()) 
      return result;
      //throw parser.eList.get(0);
    } catch (InvocationTargetException e) {
      throw e.getCause();
    }
  }
  
  public static void parseCheckResource(String a, String exp) throws Throwable {
    Object o = parseProduction("resource", a);
    if (o instanceof Resource) {
      Resource r = (Resource) o;
      String bb = new VisitorToString().run(r.expression).toString();
      assertEquals(normalize(exp), normalize(bb));
      return;
    }
    throw Assertions.codeNotReachable(" " + " " + a + " " + exp + " " + o);
  }
  public static void parseCheckClassTable(String a, int howMany) throws Throwable {
    Object o = parseProduction("classTable", a);
    if (o instanceof ClassTable) {
      ClassTable r = (ClassTable) o;
      assertEquals(r.keySet().size(), howMany);
      return;
    }
    throw Assertions.codeNotReachable(" " + " " + a + " " + howMany + " " + o);
  }
  
  public static void parseEqualToStr(String prodName, String a, String b) throws Throwable {
    Object o = parseProduction(prodName, a);
    if (o instanceof Ast) {
      Ast result = (Ast) o;
      String bb = new VisitorToString().run(result).toString();
      assertEquals(normalize(b), normalize(bb));
      return;
    }
    throw Assertions.codeNotReachable(" " + prodName + " " + a + " " + b + " " + o);
  }
  public static void parsedToEqualAst(String prodName, String a, String b) throws Throwable {
    Object o = parseProduction(prodName, a);
    Object o2 = parseProduction(prodName, b);
    assertEquals(o, o2);
  }
  public static void parsedEqualFix(String prodName, String a) throws Throwable {
    parseEqualToStr(prodName, a, a);
    parseAndRegenerateAsJava(prodName, a);
  }
  private static void parseAndRegenerateAsJava(String prodName, String a) throws Throwable {
    Eval.setDir(testTempDir, null);
    Object o = parseProduction(prodName, a);
    if (o instanceof Ast) {
      Ast result = (Ast) o;
      String bb = VisitorToAstJavaCode.run(result).toString();
      CompiledExpression compiled = Eval.compile(Ast.class.getCanonicalName() + " (){return " + bb + ";}");
      Ast result2 = (Ast) compiled.invoke();
      assertEquals(result, result2);
      return;
    }
    throw Assertions.codeNotReachable(" " + prodName + " " + a + " " + o);
    
  }
  public static void parsedEqual(String prodName, String a, String b) throws Throwable {
    parseEqualToStr(prodName, a, b);
    parsedToEqualAst(prodName, a, b);
  }
  static public String normalize(String n) {
    StringBuffer result = new StringBuffer();
    boolean lastWasSeparator = true;
    for (int i = 0; i < n.length(); i++) {
      char c = n.charAt(i);
      if (isSeparator(c)) {
        //if (!lastWasSeparator) result.append(' ');
        //lastWasSeparator = true;
      } else {
        lastWasSeparator = false;
        result.append(c);
      }
    }
    if (result.charAt(result.length() - 1) == ' ') result.deleteCharAt(result.length() - 1);
    return result.toString();
  }
  public static boolean isSeparator(char c) {
    return c == ' ' || c == '\n' || c == '\t';
  }
}

class Test2 {
  static boolean foo(int TestHelper, int Test2) {
    int[][] arr = new int[][]{new int[]{1, 2}, new int[]{3}};
    //arr.length;
    return TestHelper > 2;
  }
  Test2 Test2;
  Test2(Test2 Test2) {
    this.Test2 = Test2;
    
  }
}
