package testing;

import static org.testng.AssertJUnit.*;

import java.io.File;
import java.util.ArrayList;

import lowLevel.eval.Eval;
import lowLevel.eval.Eval.CompiledExpression;
import lowLevel.producer.JavaCompilationError;

import org.antlr.runtime.*;
import org.testng.annotations.Factory;
import org.testng.annotations.Test;

import primitiveType.MFJClass;
import primitiveType.MFJName;
import structure.astNode.Ast.BaseClassValue;
import structure.astNode.Ast.ClassName;
import structure.integrity.ClassNotWellFormed.*;
import structure.resource.YellowResource.NormalizedYellowResource;
import testUtils.TestHelper;
import tools.FileManager;
import visitors.VisitorTranslateZero;

import compiler.Mfjc;

public class TestAll {
  @Test
  public void baseClass0() throws Throwable {
    TestHelper.parsedEqualFix("e",//
        "{ Foo foo ( ) { } abstract Bar bar ( ) ;" + " constructor ( ) { } } ");
  }
  
  @Test(expectedExceptions = {NoViableAltException.class})
  public void error1() throws Throwable {
    TestHelper.parsedEqualFix("stmIf", "if ( 42 ) { } ; else { } ; ");//If
  }
  @Test(expectedExceptions = {MismatchedTokenException.class})
  public void error1b() throws Throwable {
    TestHelper.parsedEqualFix("stmIf", "if ;( 42 ) { }  else { } ; ");//If
  }
  @Test(expectedExceptions = {NoViableAltException.class})
  public void error1c() throws Throwable {
    TestHelper.parsedEqualFix("stmIf", "if ( switch ) { }  else { } ; ");//If
  }
  
  /*
  @Test(expectedExceptions = {DuplicatedClassDeclaration.class})
    public void error1() throws Throwable {
    TestHelper.parsedEqualFix("stmIf", "if ( 42 ) { } ; else { } ; ");//If
  }
  */

  @Test(expectedExceptions = {MismatchedTokenException.class})
  public void error2a() throws Throwable {
    TestHelper.parsedEqualFix("e", "{Foo foo(){}; abstract Bar foo(); constructor(){} } ");
  }
  @Test(expectedExceptions = {DuplicatedMemberName.class})
  public void error2() throws Throwable {
    TestHelper.parsedEqualFix("e", "{Foo foo(){} abstract Bar foo(); constructor(){} } ");
  }
  
  @Test(expectedExceptions = {DuplicatedParameterName.class})
  public void error3() throws Throwable {
    TestHelper.parsedEqualFix("e", "{Foo foo(Bar b, int b){} constructor( ){ } } ");
  }
  
  @Test(expectedExceptions = {DuplicatedSupertypeName.class})
  public void error4() throws Throwable {
    TestHelper.parsedEqualFix("e", "implements Foo, Bar, Foo{ constructor( ){ } } ");
  }
  @Test(expectedExceptions = {MismatchedSetException.class})
  public void error5() throws Throwable {
    TestHelper.parsedEqualFix("e", "{ abstract Foo bar(Len this); constructor( ){ } } ");
  }
  @Test(expectedExceptions = {MoreThanOneConstructor.class})
  public void error6() throws Throwable {
    TestHelper.parsedEqualFix("e",//
        "{ constructor(Err arr){} constructor( ){ } } ");
  }
  
  @Test
  //(expectedExceptions = {NoConstructor.class})//autoAdded
  public void error7() throws Throwable {
    TestHelper.parseEqualToStr("e", "{ abstract Foo bar(Len len); } ", "{ abstract Foo bar(Len len);constructor(){} } ");
    // rename of len into _len is done when all the class table is closed.
  }
  
  @Test(expectedExceptions = {UnboundInternalName.class})
  public void error8() throws Throwable {
    TestHelper.parsedEqualFix("e", "{ Foo bar(){return beer();} constructor( ){ } } ");
  }
  
  // @Test(expectedExceptions = {UndefinedVariableAccess.class})
  // public void error9() throws Throwable {
  //   TestHelper.parsedEqualFix("e", "{ Foo bar(Len x){return y;} constructor( ){ } } ");
  // }// y could be a class name
  
  @Test(expectedExceptions = {UsegeOfIntenalNameIntoConstructor.class})
  public void error10() throws Throwable {
    TestHelper.parsedEqualFix("e", "{ Foo bar; constructor(X x ){ bar=x+bar;} } ");
  }
  
  @Test(expectedExceptions = {VariableAlreadyDefined.class})
  public void error11() throws Throwable {
    TestHelper.parsedEqualFix("e", "{ Foo bar(){Foo x=1; if(true){Fee x=2;return x;}else{ }} constructor( ){ } } ");
  }
  @Test(expectedExceptions = {VariableAlreadyDefined.class})
  public void error12() throws Throwable {
    TestHelper.parsedEqualFix("e", "{ Foo bar(Foo x){Fee x=2;return x;} constructor( ){ } } ");
  }
  @Test(expectedExceptions = {NotInitializedField.class})
  public void error13() throws Throwable {
    TestHelper.parsedEqualFix("e", "{ Foo foo; constructor( ){ } } ");
  }
  @Test(expectedExceptions = {DuplicatedClassDeclaration.class})
  public void error14() throws Throwable {
    TestHelper.parseCheckClassTable("class C = 1 + 2 " //
        + "class D=C "// 
        + "class C=C+D ", 3);
  }
  @Test(expectedExceptions = {ExpressionNotStatement.class})
  public void error15a() throws Throwable {
    TestHelper.parseCheckClassTable("class C = 1 + 2 " //
        + "class D=F "// 
        + "class E=C+D "//
        + "main=1;", 3);
  }
  @Test(expectedExceptions = {ClassTableNotClosed.class})
  public void error15() throws Throwable {
    TestHelper.parseCheckClassTable("class C = 1 + 2 " //
        + "class D=F "// 
        + "class E=C+D "//
        + "main=print(1);", 3);
  }
  @Test(expectedExceptions = {NoViableAltException.class})
  public void error16() throws Throwable {
    TestHelper.parseCheckClassTable("class C = 1 + 2 " //
        + "class D={Foo int; constructor(){int=1;}}"// 
        + "class E=C+D"//
        + "main=print(1);", 3);
  }
  @Test(expectedExceptions = {MismatchedSetException.class})
  public void error18() throws Throwable {
    TestHelper.parseCheckClassTable("class C = 1 + 2 " //
        + "class D={ constructor(Foo int){}}"// 
        + "class E=C+D" //
        + "main=print(1);", 3);
  }
  @Test
  public void error19() throws Throwable {
    TestHelper.parseCheckClassTable("class C = 1 + 2 " //
        + "class D={C D; constructor(){D=1;}} "// 
        + "class E=C+D "//
        + "main=print(1);", 3);
  }
  @Test
  public void error20() throws Throwable {
    TestHelper.parseCheckClassTable("class C = 1 + 2 " //
        + "class D={ constructor(E D){}} "// 
        + "class E=C+D "//
        + "main=print(1);", 3);
  }
  @Test
  public void resources() throws Throwable {
    TestHelper.parseCheckResource("class C = { constructor ( ) { } } ", "{ constructor ( ) { } }");
    TestHelper.parseCheckResource("class D= { abstract String foo;}", "{ abstract String foo; constructor ( ) { } }");
    TestHelper.parseCheckResource("class C = 1 + 2 ", " ( 1 + 2 )");
    TestHelper.parseCheckClassTable("class C = 1 + 2 main=print(1);", 1);
    TestHelper.parseCheckClassTable("class C = 1 + 2 " //
        + "class D=C "//
        + "main=print(1);", 2);
    TestHelper.parseCheckClassTable("class C = 1 + 2 " //
        + "class D=C "// 
        + "class E=C+D "//
        + "main=print(1);", 3);
  }
  @Test
  public void ParsingTest() throws Throwable {
    TestHelper.parsedEqualFix("className", "Foo");//className
    TestHelper.parsedEqualFix("className", "int");//className
    TestHelper.parsedEqualFix("className", "class");//className
    TestHelper.parsedEqualFix("literal", "42");//IntValue
    TestHelper.parsedEqualFix("literal", "\"f2\"");//StringValue
    TestHelper.parsedEqualFix("literal", "true");//BoolValue
    TestHelper.parsedEqualFix("literal", "false");//BoolValue
    //TestHelper.parsedEqualFix("literal", "$$ Foo");//ClassNameValue
    TestHelper.parsedEqualFix("literal", "$ foo");//MemberNameValue
    TestHelper.parsedEqualFix("variableName", "foo");//VariableName
    TestHelper.parsedEqualFix("variableAccess", "foo");//VariableName
    TestHelper.parsedEqualFix("atomE", "foo");//VariableName
    TestHelper.parsedEqualFix("baseE", "foo");//VariableName
    TestHelper.parsedEqualFix("e", "foo");//VariableName
    //no obtainable directly
    //TestHelper.parsedEqualFix("e","foo");//InternalFieldAccess
    //InternalMethodCall
    TestHelper.parsedEqualFix("e", "foo . bar");//ClientFieldAccess
    TestHelper.parsedEqualFix("e", "foo . bar ( 42 , r , \"hi\" ) ");//ClientMethodCall
    //after
    //TestHelper.parsedEqualFix("e", "foo");//BaseClassValue
    TestHelper.parsedEqualFix("paramDecList", "( )");//ParamsDec
    TestHelper.parsedEqualFix("paramDecList", "( foo q )");//ParamsDec
    TestHelper.parsedEqualFix("paramDecList", "( Too qq , oo eqq )");//ParamsDec
    TestHelper.parsedEqualFix("pList", "( )");//ParamsUse
    TestHelper.parsedEqualFix("pList", "( 1 )");//ParamsUse
    TestHelper.parsedEqualFix("pList", "( 1 , 2 , true )");//ParamsUse
    TestHelper.parsedEqualFix("constructor", "constructor ( f o ) { gg = ( 3 + 2 ) ; } ");//Constructor
    TestHelper.parsedEqualFix("constructor", "constructor ( ) { gg = ( 3 + 2 ) ; hr = true ; }");//Constructor
    TestHelper.parsedEqualFix("field", " foo bar ;");//Field
    TestHelper.parsedEqualFix("abstractField", "abstract foo bar ;");//AbstractField
    TestHelper.parsedEqualFix("method", "foo bar ( ) { } ");//Method
    TestHelper.parsedEqualFix("stmBlock", "{ }");//Block
    TestHelper.parsedEqualFix("stmBlock", "{ print ( 2 ) ; }");//Block
    TestHelper.parsedEqualFix("stmBlock", "{ print ( 2 ) ; return 1 ; }");//Block
    TestHelper.parsedEqualFix("stmIf", "if ( true ) return 1 ; else print ( 2 ) ;");//If
    TestHelper.parsedEqualFix("stmIf", "if ( 42 ) { } else { } ");//If
    TestHelper.parsedEqualFix("stmIf", "if ( 42 ) { } else { return 11 ; }");//If
    TestHelper.parsedEqualFix("stmWhile", "while ( true ) { print ( false ) ; } ");//While
    TestHelper.parsedEqualFix("stmDec", "foo bar = 3 ;");//VariableDeclaration
    TestHelper.parsedEqualFix("stmReturn", "return 1 ;");//Return
    TestHelper.parsedEqualFix("abstractMethod", "abstract foo bar ( ) ;");//AbstractMethod
    TestHelper.parsedEqualFix("atomE", " ( 1 + ( 2 * 4 ) )");//(e)
    TestHelper.parsedEqualFix("atomE", "new C ( )");//new
    TestHelper.parsedEqualFix("atomE", "new C ( 1 , 2 )");//new
    TestHelper.parsedEqualFix("stmPrint", "print ( 2 ) ;");//Print
    TestHelper.parseEqualToStr("e", "( 1 + 2 * 3 )", "( 1 + ( 2 * 3 ) )");//BinOpExp
    TestHelper.parseEqualToStr("e", "1==1 && \"ciao\" ==\"ciao\" && $ciao ==$ciao", "(((1==1) && (\"ciao\" ==\"ciao\")) && ($ciao ==$ciao))");
    TestHelper.parsedEqualFix("e", "( ( 1 + 2 ) * 3 )");//BinOpExp
    TestHelper.parsedEqualFix("e", "( 1 + ( 2 * 3 ) )");//BinOpExp
    TestHelper.parsedEqualFix("e", " ! false ");//UnOpExp
    TestHelper.parsedEqualFix("e", " ! ! false");//UnOpExp
    TestHelper.parsedEqualFix("e", "! ! ! false");//UnOpExp
    TestHelper.parsedEqualFix("e", "foo [ restrict 2 ]");//RestrictExp
    TestHelper.parsedEqualFix("e", "foo [ alias 2 to 3 ]");//AliasExp
    TestHelper.parsedEqualFix("e", "foo [ redirect 3 to 2 ]");//RedirectExp
    
    TestHelper.parsedEqualFix("e", "foo [ members ]");//RedirectExp
    TestHelper.parsedEqualFix("e", "foo [ 3 ]");//RedirectExp
    TestHelper.parsedEqualFix("e", "new foo[][]{1,2,new bar[]{}}");//new array
    TestHelper.parsedEqualFix("stm", "foo.bar(2);");//client
    TestHelper.parsedEqualFix("stm", "foo.doo.bar(2);");//nested
    TestHelper.parsedEqualFix("stm", "(1+2).bar(2);");//nested
    TestHelper.parsedEqualFix("stm", "bar(2);");//internal
    TestHelper.parsedEqualFix("stm", "new Foo();");//constructor
    TestHelper.parsedEqualFix("stm", "throw 2;");//throw
    TestHelper.parsedEqualFix("stm", //
        "try{bar(2,3);}catch(T x){}catch(R r){print(1);}");//try
    TestHelper.parsedEqualFix("stmForeach", //
        "for(int i:bar(2,3))print((1+2));");//foreach
    TestHelper.parsedEqualFix("stm", //
        "for(int i:bar(2,3))print((1+2));");//foreach
  }
  
  @Test(expectedExceptions = {JavaCompilationError.class})
  public void evalError0() throws Throwable {
    Eval.setDir(TestHelper.testTempDir, null);
    FileManager.deleteDir(new File(TestHelper.testTempDir, Eval.packageName));
    CompiledExpression c = Eval.compile("int (){return null;}");
  }
  @Test
  public void eval0() throws Throwable {
    Eval.setDir(TestHelper.testTempDir, null);
    FileManager.deleteDir(new File(TestHelper.testTempDir, Eval.packageName));
    CompiledExpression c = Eval.compile("int (){return 1;}");
    int i = (Integer) c.invoke();
    assertEquals(1, i);
    c = Eval.compile("int (int x){return x*2;}");
    i = (Integer) c.invoke(3);
    assertEquals(6, i);
  }
  
  @Test
  public void baseTypes() throws Throwable {
    assertEquals("int ", new VisitorTranslateZero(null).run(new ClassName("int")).toString());
    assertEquals("boolean ", new VisitorTranslateZero(null).run(new ClassName("boolean")).toString());
    assertEquals("String ", new VisitorTranslateZero(null).run(new ClassName("String")).toString());
    assertEquals("Object ", new VisitorTranslateZero(null).run(new ClassName("Object")).toString());
    assertEquals("RuntimeException ", new VisitorTranslateZero(null).run(new ClassName("RuntimeException")).toString());
    assertEquals(MFJClass.class.getCanonicalName() + " ", new VisitorTranslateZero(null).run(new ClassName("class")).toString());
    assertEquals(MFJName.class.getCanonicalName() + " ", new VisitorTranslateZero(null).run(new ClassName("name")).toString());
  }
  @Factory
  public static Object[] createInstances() {
    ArrayList<Object> result = new ArrayList<Object>();
    for (File f : TestHelper.collectTestFiles())
      result.add(new FileTest(f.getName(), f));
    return result.toArray();
  }
  
  @Test
  public static class FileTest {
    public FileTest() {
      assert false;
    }
    public FileTest(String fileName, File f) {
      this.fileName = fileName;
      this.f = f;
    }
    String fileName;
    File f;
    
    @Override
    public String toString() {
      return fileName;
    }
    @Test
    //(testName = "a")
    void runTest() throws Throwable {
      Mfjc.setDir(TestHelper.testTempDir, null);
      Eval.setDir(TestHelper.testTempDir, Mfjc.getLoader());
      FileManager.deleteDir(new File(TestHelper.testTempDir, Eval.packageName));
      FileManager.deleteDir(new File(TestHelper.testTempDir, "" + Mfjc.programCode));
      String firstLine = FileManager.readFirstLine(f);
      System.out.println("START TEST ON  " + f.getAbsolutePath());
      VisitorTranslateZero.generationNumber++;
      System.out.println("test start");
      Mfjc.compileFile(f);
      if (firstLine.equals("//#")) {
        System.out.println("verification start");
        Class<?> mainClass = FileManager.loadClassDiskWait(VisitorTranslateZero.cName("Main"), Mfjc.getLoader());
        int result = (Integer) mainClass.getMethod("main").invoke(mainClass.newInstance());
        if (result != 1) throw new Error("Unespected result value test " + f.getName() + " " + result);
        System.out.println("test  " + f.getName() + "    " + result);
      } else System.out.println("test " + f.getName() + " verification not required");
    }
  }
  
  @Test
  public void ty1() throws Throwable {
    assert TestHelper.isSeparator(' ');
    assert TestHelper.isSeparator('\n');
    assert TestHelper.isSeparator('\t');
    assertEquals(TestHelper.normalize(" foo   bar r \n    \t  53 e3;"), "foobarr53e3;");
    assertEquals(TestHelper.normalize(//
        "  foo  " + //
            "  bar r;"), "foobarr;");
    TestHelper.parseEqualToStr("stmIf", "if(42)return 1;else {}", "if ( 42 ) return 1 ; else { }");
  }
  @Test
  public void t2() throws Throwable {
    TestHelper.parseEqualToStr("stmIf", "if(42)return 1;else {}", "if ( 42 ) return 1 ; else { }");
    TestHelper.parsedToEqualAst("stmIf", "if(42)return 1;else {}", "if ( 42 ) return 1 ; else { }");
    TestHelper.parsedEqualFix("stmIf", "if ( 42 ) return 1 ; else { }");
    TestHelper.parsedEqual("stmIf", "if(42)return 1;else {}", "if ( 42 ) return 1 ; else { }");
  }
  @Test(expectedExceptions = {AssertionError.class})
  public void t3() throws Throwable {
    TestHelper.parseEqualToStr("stmIf", "if(42)return 1;else {}", "if ( 42 ) return 1 ; else { return 1; }");
  }
  @Test(expectedExceptions = {AssertionError.class})
  public void t4() throws Throwable {
    TestHelper.parsedToEqualAst("e", "1+(2+3)", "(1+2)+3");
  }
  //@Test(expectedExceptions = {AssertionError.class})
  /*  public void t5() throws Throwable {
      TestHelper.parsedEqualFix("stmIf", "if( 42 ) return 1 ; else { }");
    }*/
  @Test(expectedExceptions = {AssertionError.class})
  public void t6() throws Throwable {
    TestHelper.parsedEqual("stmIf", "if(42)return 1;else {}", "if ( 42 ) return 1 ; else { return 1;}");
  }
  
  NormalizedYellowResource data(String name, String ast) throws Throwable {
    BaseClassValue bc = (BaseClassValue) TestHelper.parseProduction("bc", ast);
    return new NormalizedYellowResource(new ClassName(name), bc);
  }
  
  /* public static void checkEquals(String code, NormalizedYellowResource... rs) {
     HashMap<String, StringBuffer> tmp = TranslateStar.run(AL(rs));
     assertEquals(TestHelper.normalize(tmp.toString()), TestHelper.normalize(code));
   }*/
  /* Too Fragile
  @Test
  public void tx1() throws Throwable {
    checkEquals(//
        "{iPippo13=public interface iPippo13{int foo();}," + //
            " cPippo13=public classc Pippo13 implements programCode.iPippo13{" + //
            "  public cPippo13(){} public int foo(){return1;}" + //
            "  public static structure.astNode.Ast.BaseClassValue __private__ThisAst" + //
            "    =newstructure.astNode.Ast.BaseClassValue(" + //
            "      structure.astNode.Sequence.hsClassName()," + //
            "      structure.astNode.Sequence.mMember(" + //
            "        newstructure.astNode.Ast.Method(" + //
            "          newstructure.astNode.Ast.ClassName(\"int\")," + //
            "          newstructure.astNode.Ast.MemberName(\"foo\")," + //
            "          newstructure.astNode.Ast.ParamsDec(" + //
            "            structure.astNode.Sequence.alClassName()," + //
            "            structure.astNode.Sequence.alVariableName()" + //
            "            )," + //
            "          newstructure.astNode.Ast.Block(" + //
            "            structure.astNode.Sequence.alStatement(" + //
            "              newstructure.astNode.Ast.Return(" + //
            "                newstructure.astNode.Ast.IntValue(1)" + //
            "                )" + //
            "              )" + //
            "            )" + //
            "          )" + //
            "        )," + //
            "      newstructure.astNode.Ast.Constructor(" + //
            "        newstructure.astNode.Ast.ParamsDec(" + //
            "          structure.astNode.Sequence.alClassName()," + //
            "          structure.astNode.Sequence.alVariableName()" + //
            "          )," + //
            "        structure.astNode.Sequence.alMemberName()," + //
            "        structure.astNode.Sequence.alExpression()" + //
            "        )" + //
            "      );" + //
            "    }" + //
            "  }",//
        
        data("Pippo",//
            "{constructor(){} int foo(){return 1;}}"));
  }
  */
  /* Too Fragile 
   * @Test
    public void tx2() throws Throwable {
      checkEquals(//
          " { " + //
              " cPippo0=public class cPippo0 implements programCode.iPippo0 { " + //
              " public cPippo0 ( ) { foo=1;} " + //
              " private int foo;" + //
              " public int foo ( ) { return foo ; } " + //
              " } " + //
              " , " + //
              " iPippo0=public interface iPippo0 { " + //
              " int foo ( ) ; " + //
              " }" + //
              " } ",//
          data("Pippo",//
              "{constructor(){foo =1;} int foo;}"));
    }*/
}
