package tools;

import static tools.Assertions.*;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.NotSerializableException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.lang.reflect.Array;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Arrays;

public class Cloning {
  @SuppressWarnings("unchecked")
  public final static <C> C copy(C c) {
    try {
      Class<?> clazz = c.getClass();
      if (clazz.isArray()) // even though arrays implicitly have a public clone(), it cannot be invoked reflectively
        return (C) Arrays.copyOf((Object[]) c, Array.getLength(c));
      Method m = clazz.getMethod("clone");
      return (C) m.invoke(c);
    } catch (SecurityException e) {
      reflectionError(e.toString(), e);
    } catch (NoSuchMethodException e) {
      reflectionError(e.toString(), e);
    } catch (IllegalArgumentException e) {
      reflectionError(e.toString(), e);
    } catch (IllegalAccessException e) {
      reflectionError(e.toString(), e);
    } catch (InvocationTargetException e) {
      reflectionError(e.toString(), e);
    }
    codeNotReachable();
    return null;
  }
  
  @SuppressWarnings("unchecked")
  public final static <C extends Serializable> C deepCopy(C c) {
    ByteArrayOutputStream buf = new ByteArrayOutputStream();
    try {
      ObjectOutputStream o = new ObjectOutputStream(buf);
      o.writeObject(c);
      ObjectInputStream in = new ObjectInputStream(new ByteArrayInputStream(buf.toByteArray()));
      return (C) in.readObject();
    } catch (NotSerializableException e) {
      userErrorAssert(e.toString(), e);
    } catch (IOException e) {
      systemError(e.toString(), e);
    } catch (ClassNotFoundException e) {
      systemError(e.toString(), e);
    }
    codeNotReachable();
    return null;
  }
  
}
