package tools;

import java.io.Serializable;
import java.util.*;

public class MultiContainer<K, T> implements Serializable {
  private HashMap<K, T> map = new HashMap<K, T>();
  
  public Set<K> keySet() {
    return map.keySet();
  }
  
  public void put(K key, T value) {
    map.put(key, value);
  }
  public <C extends T> C get(Class<C> clazz, K key) {
    assert isContained(clazz, key);
    T result = map.get(key);
    return (C) result;
  }
  public <C extends T> boolean isContained(Class<C> clazz, K key) {
    if (!map.containsKey(key)) return false;
    T result = map.get(key);
    return clazz.isAssignableFrom(result.getClass());
  }
  public <C extends T> void remove(Class<C> clazz, K key) {
    assert isContained(clazz, key);
    map.remove(key);
  }
  
  public Collection<T> values() {
    return map.values();
  }
  public <C extends T> Set<K> keySet(Class<C> clazz) {
    HashSet<K> result = new HashSet<K>();
    for (K key : keySet())
      if (isContained(clazz, key)) result.add(key);
    return result;
  }
  public <C extends T> Collection<C> values(Class<C> clazz) {
    ArrayList<C> result = new ArrayList<C>();
    for (T e : values())
      if (clazz.isAssignableFrom(e.getClass())) result.add((C) e);
    return result;
  }
}
