package visitors;

import java.util.Collection;
import java.util.HashSet;

import org.antlr.runtime.RecognitionException;

import structure.astNode.Ast.*;
import structure.integrity.ClassNotWellFormed.VariableAlreadyDefined;
import tools.DataStructures.WrapperException;
public class VisitorCheckNoVariablesDeclaredTwiceZero extends VisitorReader {
  HashSet<VariableName> defined;
  static final VariableName thisVarName = new VariableName("this");
  
  public static void run(Statement e) {
    VisitorCheckNoVariablesDeclaredTwiceZero vv = new VisitorCheckNoVariablesDeclaredTwiceZero(new HashSet<VariableName>());
    e.accept(vv);
  }
  
  public static void run(BaseClassValue e) throws RecognitionException {
    try {
      VisitorCheckNoVariablesDeclaredTwiceZero vv = new VisitorCheckNoVariablesDeclaredTwiceZero();
      for (Member m : e.members.values())
        m.accept(vv);
      e.constructor.accept(vv);
    } catch (WrapperException t) {
      throw (RecognitionException) t.wrappedElement;
    }
  }
  
  @Override
  public NoResult visit(BaseClassValue e) {
    return noResult;
  }
  private VisitorCheckNoVariablesDeclaredTwiceZero(Collection<VariableName> defined) {
    super();
    this.defined = new HashSet<VariableName>(defined);
  }
  private VisitorCheckNoVariablesDeclaredTwiceZero() {
    super();
    this.defined = null;
  }
  
  public NoResult superVisit(Constructor e) {
    return super.visit(e);
  }
  @Override
  public NoResult visit(Constructor e) {
    return new VisitorCheckNoVariablesDeclaredTwiceZero(//
        e.params.names).superVisit(e);
  }
  public NoResult superVisit(Method e) {
    return super.visit(e);
  }
  @Override
  public NoResult visit(Method e) {
    HashSet<VariableName> def = new HashSet<VariableName>(e.params.names);
    def.add(thisVarName);
    return new VisitorCheckNoVariablesDeclaredTwiceZero(//
        def).superVisit(e);
  }
  
  public NoResult superVisit(Block e) {
    return super.visit(e);
  }
  @Override
  public NoResult visit(Block e) {
    return new VisitorCheckNoVariablesDeclaredTwiceZero(//
        defined).superVisit(e);
  }
  
  @Override
  public NoResult visit(VariableDeclaration e) {
    if (defined.contains(e.name)) throw new WrapperException(new VariableAlreadyDefined(e.name.value));
    defined.add(e.name);
    return super.visit(e);
  }
  /*@Override
  public NoResult visit(VariableAccess e) {
    if (!defined.contains(e.value)) throw new WrapperException(new UndefinedVariableAccess(e.value.value));
    return super.visit(e);
  }*/

  @Override
  public NoResult visit(Try e) {
    visit(e._try);
    assert (e.types.size() == e.names.size()) && (e.types.size() == e.blocks.size());
    for (int i = 0; i < e.types.size(); i++) {
      if (defined.contains(e.names.get(i))) throw new WrapperException(new VariableAlreadyDefined(e.names.get(i).value));
      HashSet<VariableName> def = new HashSet<VariableName>(defined);
      def.add(e.names.get(i));
      new VisitorCheckNoVariablesDeclaredTwiceZero(//
          def).superVisit(e.blocks.get(i));
    }
    return noResult;
  }
  @Override
  public NoResult visit(Foreach e) {
    e.expression.accept(this);
    if (defined.contains(e.name)) throw new WrapperException(new VariableAlreadyDefined(e.name.value));
    HashSet<VariableName> def = new HashSet<VariableName>(defined);
    def.add(e.name);
    e.body.accept(new VisitorCheckNoVariablesDeclaredTwiceZero(def));
    return noResult;
  }
}
