package visitors;

import java.util.ArrayList;
import java.util.HashSet;

import structure.astNode.Ast;
import structure.astNode.Members;
import structure.astNode.Ast.*;
import structure.astNode.Ast.StringValue;

public class VisitorClone implements Visitor<Ast> {
  
  public <T extends Ast> ArrayList<T> visitCollection(ArrayList<T> list) {
    ArrayList<T> newList = new ArrayList<T>();
    boolean modList = false;
    for (T e : list) {
      T ee = (T) e.accept(this);
      if (!ee.equals(e)) modList = true;
      newList.add(ee);
    }
    if (modList) return newList;
    return list;
  }
  public <T extends Ast> HashSet<T> visitCollection(HashSet<T> list) {
    HashSet<T> newList = new HashSet<T>();
    boolean modList = false;
    for (T e : list) {
      T ee = (T) e.accept(this);
      if (!ee.equals(e)) modList = true;
      newList.add(ee);
    }
    if (modList) return newList;
    return list;
  }
  
  public IntValue visit(IntValue e) {
    return e;
  }
  public StringValue visit(StringValue e) {
    return e;
  }
  public BoolValue visit(BoolValue e) {
    return e;
  }
  public ClassNameValue visit(ClassNameValue e) {
    return e;
  }
  public MemberNameValue visit(MemberNameValue e) {
    return e;
  }
  public ClassName visit(ClassName e) {
    return e;
  }
  public MemberName visit(MemberName e) {
    return e;
  }
  public VariableName visit(VariableName e) {
    return e;
  }
  public Expression visit(VariableAccess e) {
    VariableName v = visit(e.value);
    if (v.equals(e.value)) return e;
    return new VariableAccess(v);
  }
  public Expression visit(InternalFieldAccess e) {
    MemberName v = visit(e.value);
    if (v.equals(e.value)) return e;
    return new InternalFieldAccess(v);
  }
  public Expression visit(InternalMethodCall e) {
    MemberName v = visit(e.value);
    ParamsUse p = visit(e.params);
    if (v.equals(e.value) && p.equals(e.params)) return e;
    return new InternalMethodCall(v, p);
  }
  public Expression visit(ClientFieldAccess e) {
    Ast r = e.reciver.accept(this);
    MemberName v = visit(e.value);
    if (r.equals(e.reciver) && v.equals(e.value)) return e;
    return new ClientFieldAccess((Expression) r, v);
  }
  public Expression visit(ClientMethodCall e) {
    Ast r = e.reciver.accept(this);
    MemberName v = visit(e.value);
    ParamsUse p = visit(e.params);
    if (r.equals(e.reciver) && v.equals(e.value) && p.equals(e.params)) return e;
    return new ClientMethodCall((Expression) r, v, p);
  }
  
  public BaseClassValue visit(BaseClassValue e) {
    HashSet<ClassName> newImpl = visitCollection(e.impl);
    boolean modMembers = false;
    Members newMembers = new Members();
    for (Member m : e.members.values()) {
      Member mm = (Member) m.accept(this);
      if (!mm.equals(m)) modMembers = true;
      newMembers.put(mm.getName(), mm);
    }
    if (!modMembers) newMembers = e.members;
    
    Constructor cc = visit(e.constructor);
    if (newImpl == e.impl && !modMembers && cc.equals(e.constructor)) return e;
    return new BaseClassValue(newImpl, newMembers, cc);
  }
  public ParamsDec visit(ParamsDec e) {
    ArrayList<VariableName> names = visitCollection(e.names);
    ArrayList<ClassName> types = visitCollection(e.types);
    if (names == e.names && types == e.types) return e;
    return new ParamsDec(types, names);
  }
  public ParamsUse visit(ParamsUse e) {
    ArrayList<Expression> actual = visitCollection(e.actual);
    if (actual == e.actual) return e;
    return new ParamsUse(actual);
  }
  public Constructor visit(Constructor e) {
    ParamsDec p = visit(e.params);
    ArrayList<MemberName> names = visitCollection(e.names);
    ArrayList<Expression> expressions = visitCollection(e.expressions);
    if (p.equals(e.params) && expressions == e.expressions && names == e.names) return e;
    return new Constructor(p, names, expressions);
  }
  public Field visit(Field e) {
    ClassName type = visit(e.type);
    MemberName name = visit(e.name);
    if (type.equals(e.type) && name.equals(e.name)) return e;
    return new Field(type, name);
  }
  public AbstractField visit(AbstractField e) {
    ClassName type = visit(e.type);
    MemberName name = visit(e.name);
    if (type.equals(e.type) && name.equals(e.name)) return e;
    return new AbstractField(type, name);
  }
  public Method visit(Method e) {
    ClassName returnType = visit(e.returnType);
    MemberName name = visit(e.name);
    ParamsDec p = visit(e.params);
    Block b = visit(e.body);
    if (returnType.equals(e.returnType) && name.equals(e.name) && p.equals(e.params) && b.equals(e.body)) return e;
    return new Method(returnType, name, p, b);
  }
  public Block visit(Block e) {
    ArrayList<Statement> stms = visitCollection(e.statements);
    if (stms == e.statements) return e;
    return new Block(stms);
  }
  public If visit(If e) {
    Expression c = (Expression) e.condition.accept(this);
    Statement then = (Statement) e.then.accept(this);
    Statement _else = (Statement) e._else.accept(this);
    if (c.equals(e.condition) && then.equals(e.then) && _else.equals(e._else)) return e;
    return new If(c, then, _else);
  }
  public While visit(While e) {
    Expression c = (Expression) e.condition.accept(this);
    Statement body = (Statement) e.body.accept(this);
    if (c.equals(e.condition) && body.equals(e.body)) return e;
    return new While(c, body);
  }
  public VariableDeclaration visit(VariableDeclaration e) {
    ClassName type = visit(e.type);
    VariableName name = visit(e.name);
    Expression init = (Expression) e.initialization.accept(this);
    if (type.equals(e.type) && name.equals(e.name) && init.equals(e.initialization)) return e;
    return new VariableDeclaration(type, name, init);
  }
  public Return visit(Return e) {
    Expression ee = (Expression) e.expression.accept(this);
    if (ee.equals(e.expression)) return e;
    return new Return(ee);
  }
  public AbstractMethod visit(AbstractMethod e) {
    ClassName returnType = visit(e.returnType);
    MemberName name = visit(e.name);
    ParamsDec p = visit(e.params);
    if (returnType.equals(e.returnType) && name.equals(e.name) && p.equals(e.params)) return e;
    return new AbstractMethod(returnType, name, p);
  }
  public New visit(New e) {
    ClassName type = visit(e.type);
    ParamsUse p = visit(e.params);
    if (type.equals(e.type) && p.equals(e.params)) return e;
    return new New(type, p);
  }
  public Print visit(Print e) {
    Expression ee = (Expression) e.expression.accept(this);
    if (ee.equals(e.expression)) return e;
    return new Print(ee);
  }
  public Expression visit(BinOpExp e) {
    Expression left = (Expression) e.left.accept(this);
    Expression right = (Expression) e.right.accept(this);
    if (left.equals(e.left) && right.equals(e.right)) return e;
    return new BinOpExp(left, e.op, right);
  }
  public Expression visit(UnOpExp e) {
    Expression inner = (Expression) e.inner.accept(this);
    if (inner.equals(e.inner)) return e;
    return new UnOpExp(e.op, inner);
  }
  public Expression visit(RestrictExp e) {
    Expression targhet = (Expression) e.targhet.accept(this);
    Expression inner = (Expression) e.inner.accept(this);
    if (targhet.equals(e.targhet) && inner.equals(e.inner)) return e;
    return new RestrictExp(targhet, inner);
  }
  public Expression visit(AliasExp e) {
    Expression targhet = (Expression) e.targhet.accept(this);
    Expression left = (Expression) e.left.accept(this);
    Expression right = (Expression) e.right.accept(this);
    if (targhet.equals(e.targhet) && left.equals(e.left) && right.equals(e.right)) return e;
    return new AliasExp(targhet, left, right);
  }
  public Expression visit(RedirectExp e) {
    Expression targhet = (Expression) e.targhet.accept(this);
    Expression left = (Expression) e.left.accept(this);
    Expression right = (Expression) e.right.accept(this);
    if (targhet.equals(e.targhet) && left.equals(e.left) && right.equals(e.right)) return e;
    return new RedirectExp(targhet, left, right);
  }
  @Override
  public Ast visit(NewArray e) {
    ClassName type = visit(e.type);
    ParamsUse params = visit(e.params);
    if (type.equals(e.type) && params.equals(e.params)) return e;
    return new NewArray(type, params);
  }
  @Override
  public Ast visit(StmExp e) {
    Expression expression = (Expression) e.expression.accept(this);
    if (expression.equals(e.expression)) return e;
    return new StmExp(expression);
  }
  @Override
  public Ast visit(VariableAssignament e) {
    VariableName name = visit(e.name);
    Expression assignedValue = (Expression) e.assignedValue.accept(this);
    if (name.equals(e.name) && assignedValue.equals(e.assignedValue)) return e;
    return new VariableAssignament(name, assignedValue);
  }
  @Override
  public Ast visit(Throw e) {
    Expression expression = (Expression) e.expression.accept(this);
    if (expression.equals(e.expression)) return e;
    return new Throw(expression);
  }
  @Override
  public Ast visit(AccessArray e) {
    Expression targhet = (Expression) e.targhet.accept(this);
    Expression inner = (Expression) e.inner.accept(this);
    if (targhet.equals(e.targhet) && inner.equals(e.inner)) return e;
    return new AccessArray(targhet, inner);
    
  }
  @Override
  public Ast visit(GetMembers e) {
    Expression targhet = (Expression) e.targhet.accept(this);
    if (targhet.equals(e.targhet)) return e;
    return new GetMembers(targhet);
  }
  @Override
  public Ast visit(Try e) {
    Block _try = visit(e._try);
    ArrayList<ClassName> types = visitCollection(e.types);
    ArrayList<VariableName> names = visitCollection(e.names);
    ArrayList<Block> blocks = visitCollection(e.blocks);
    if (_try.equals(e._try) && types == e.types && names == e.names && blocks == e.blocks) return e;
    return new Try(_try, types, names, blocks);
  }
  @Override
  public Ast visit(Foreach e) {
    ClassName type = visit(e.type);
    VariableName name = visit(e.name);
    Expression expression = (Expression) e.expression.accept(this);
    Statement body = (Statement) e.body.accept(this);
    if (type.equals(e.type) //
        && name.equals(e.name)// 
        && expression.equals(e.expression)//
        && body.equals(e.body)) return e;
    return new Foreach(type, name, expression, body);
  }
}