package visitors;

import static tools.DataStructures.*;

import java.util.*;

import org.antlr.runtime.RecognitionException;

import structure.astNode.Members;
import structure.astNode.Ast.*;
import structure.integrity.ClassNotWellFormed.UnboundInternalName;
import tools.Assertions;
import tools.DataStructures.Tuple2;
public class VisitorCompleteIternalNamesZero extends VisitorClone {
  Set<String> fields;
  Set<String> methods;
  
  public static Tuple2<Members, Constructor> run(Members m, Constructor c) throws RecognitionException {
    VisitorCompleteIternalNamesZero vv = new VisitorCompleteIternalNamesZero(//
        unwrap(m.keySet(IField.class)),//
        unwrap(m.keySet(IMethod.class))//
    );//
    Members newM = new Members();
    try {
      for (Member mm : m.values()) {
        mm = (Member) mm.accept(vv);
        newM.put(mm.getName(), mm);
      }
      
      c = vv.visit(c);
    } catch (WrapperException t) {
      throw (RecognitionException) t.wrappedElement;
    }
    return T(newM, c);
  }
  static <T extends IdHolder> Set<String> remove(Set<String> start, Collection<T> toRemove) {
    HashSet<String> result = new HashSet<String>(start);
    result.removeAll(unwrap(toRemove));
    return result;
  }
  static <T extends IdHolder> Set<String> unwrap(Collection<T> start) {
    HashSet<String> result = new HashSet<String>();
    for (T t : start)
      result.add(t.value);
    return result;
  }
  
  static Set<String> remove(Set<String> start, String toRemove) {
    HashSet<String> result = new HashSet<String>(start);
    result.remove(toRemove);
    return result;
  }
  
  public VisitorCompleteIternalNamesZero(Set<String> fields, Set<String> methods) {
    super();
    this.fields = fields;
    this.methods = methods;
  }
  
  @Override
  public BaseClassValue visit(BaseClassValue e) {
    return e;
  }
  public Constructor superVisit(Constructor e) {
    return super.visit(e);
  }
  @Override
  public Constructor visit(Constructor e) {
    return new VisitorCompleteIternalNamesZero(//
        remove(fields, e.params.names),//
        remove(methods, e.params.names)//
    ).superVisit(e);
  }
  public Method superVisit(Method e) {
    return super.visit(e);
  }
  @Override
  public Method visit(Method e) {
    return new VisitorCompleteIternalNamesZero(//
        remove(fields, e.params.names),//
        remove(methods, e.params.names)//
    ).superVisit(e);
  }
  
  public Block superVisit(Block e) {
    return super.visit(e);
  }
  @Override
  public Block visit(Block e) {
    return new VisitorCompleteIternalNamesZero(//
        fields,//
        methods//
    ).superVisit(e);
  }
  
  @Override
  public VariableDeclaration visit(VariableDeclaration e) {
    fields.remove(e.name.value);
    methods.remove(e.name.value);
    return super.visit(e);
  }
  
  @Override
  public Expression visit(VariableAccess e) {
    if (!fields.contains(e.value.value)) return super.visit(e);
    return new InternalFieldAccess(new MemberName(e.value.value));
  }
  
  @Override
  public InternalFieldAccess visit(InternalFieldAccess e) {
    throw Assertions.codeNotReachable();
    //InternalFieldAccesses do not exists at this time
    
  }
  @Override
  public Expression visit(InternalMethodCall e) {
    if (!methods.contains(e.value.value)) throw new WrapperException(new UnboundInternalName(e.value.value));
    return super.visit(e);
  }
  @Override
  public Try visit(Try e) {
    Block _try = visit(e._try);
    ArrayList<Block> blocks = new ArrayList<Block>();
    assert (e.types.size() == e.names.size()) && (e.types.size() == e.blocks.size());
    for (int i = 0; i < e.types.size(); i++) {
      blocks.add(new VisitorCompleteIternalNamesZero(//
          remove(fields, e.names.get(i).value),//
          remove(methods, e.names.get(i).value)//
          ).superVisit(e.blocks.get(i)));
    }
    if (blocks.equals(e.blocks)) blocks = e.blocks;
    if (_try.equals(e._try) && blocks == e.blocks) return e;
    return new Try(_try, e.types, e.names, blocks);
  }
  @Override
  public Foreach visit(Foreach e) {
    Expression expression = (Expression) e.expression.accept(this);
    Statement body = (Statement) e.body.accept(//
        new VisitorCompleteIternalNamesZero(//
            remove(fields, e.name.value),//
            remove(methods, e.name.value)//
        ));
    if (expression.equals(e.expression) && body.equals(e.body)) return e;
    return new Foreach(e.type, e.name, expression, body);
  }
}
