package visitors;

import structure.astNode.Ast.*;
import structure.astNode.Ast.StringValue;

public class VisitorReader implements Visitor<VisitorReader.NoResult> {
  public static class NoResult {
    private NoResult() {}
  }
  public static final NoResult noResult = new NoResult();
  public NoResult visit(IntValue e) {
    return noResult;
  }
  public NoResult visit(StringValue e) {
    return noResult;
  }
  public NoResult visit(BoolValue e) {
    return noResult;
  }
  public NoResult visit(ClassNameValue e) {
    return noResult;
  }
  public NoResult visit(MemberNameValue e) {
    return noResult;
  }
  public NoResult visit(ClassName e) {
    return noResult;
  }
  public NoResult visit(MemberName e) {
    return noResult;
  }
  public NoResult visit(VariableName e) {
    return noResult;
  }
  public NoResult visit(VariableAccess e) {
    return visit(e.value);
  }
  public NoResult visit(InternalFieldAccess e) {
    return visit(e.value);
  }
  public NoResult visit(InternalMethodCall e) {
    visit(e.value);
    visit(e.params);
    return noResult;
  }
  public NoResult visit(ClientFieldAccess e) {
    e.reciver.accept(this);
    visit(e.value);
    return noResult;
  }
  public NoResult visit(ClientMethodCall e) {
    e.reciver.accept(this);
    visit(e.value);
    visit(e.params);
    return noResult;
  }
  public NoResult visit(BaseClassValue e) {
    for (ClassName n : e.impl)
      visit(n);
    for (Member m : e.members.values())
      m.accept(this);
    visit(e.constructor);
    return noResult;
  }
  public NoResult visit(ParamsDec e) {
    for (VariableName n : e.names)
      visit(n);
    for (ClassName n : e.types)
      visit(n);
    return noResult;
  }
  public NoResult visit(ParamsUse e) {
    for (Expression ee : e.actual)
      ee.accept(this);
    return noResult;
  }
  public NoResult visit(Constructor e) {
    visit(e.params);
    for (MemberName n : e.names)
      visit(n);
    for (Expression ee : e.expressions)
      ee.accept(this);
    return noResult;
  }
  public NoResult visit(Field e) {
    visit(e.type);
    visit(e.name);
    return noResult;
  }
  public NoResult visit(AbstractField e) {
    visit(e.type);
    visit(e.name);
    return noResult;
  }
  public NoResult visit(Method e) {
    visit(e.returnType);
    visit(e.name);
    visit(e.params);
    visit(e.body);
    return noResult;
  }
  public NoResult visit(Block e) {
    for (Statement s : e.statements)
      s.accept(this);
    return noResult;
  }
  public NoResult visit(If e) {
    e.condition.accept(this);
    e.then.accept(this);
    e._else.accept(this);
    return noResult;
  }
  public NoResult visit(While e) {
    e.condition.accept(this);
    e.body.accept(this);
    return noResult;
  }
  public NoResult visit(VariableDeclaration e) {
    visit(e.type);
    visit(e.name);
    e.initialization.accept(this);
    return noResult;
  }
  public NoResult visit(Return e) {
    e.expression.accept(this);
    return noResult;
  }
  public NoResult visit(AbstractMethod e) {
    visit(e.returnType);
    visit(e.name);
    visit(e.params);
    return noResult;
  }
  public NoResult visit(New e) {
    visit(e.type);
    visit(e.params);
    return noResult;
  }
  public NoResult visit(Print e) {
    return e.expression.accept(this);
  }
  public NoResult visit(BinOpExp e) {
    e.left.accept(this);
    e.right.accept(this);
    return noResult;
  }
  public NoResult visit(UnOpExp e) {
    return e.inner.accept(this);
  }
  public NoResult visit(RestrictExp e) {
    e.targhet.accept(this);
    e.inner.accept(this);
    return noResult;
  }
  public NoResult visit(AliasExp e) {
    e.targhet.accept(this);
    e.left.accept(this);
    e.right.accept(this);
    return noResult;
  }
  public NoResult visit(RedirectExp e) {
    e.targhet.accept(this);
    e.left.accept(this);
    e.right.accept(this);
    return noResult;
  }
  @Override
  public NoResult visit(NewArray e) {
    visit(e.type);
    visit(e.params);
    return noResult;
  }
  @Override
  public NoResult visit(StmExp e) {
    return e.expression.accept(this);
  }
  @Override
  public NoResult visit(VariableAssignament e) {
    visit(e.name);
    e.assignedValue.accept(this);
    return noResult;
    
  }
  @Override
  public NoResult visit(Throw e) {
    return e.expression.accept(this);
  }
  @Override
  public NoResult visit(AccessArray e) {
    e.targhet.accept(this);
    e.inner.accept(this);
    return noResult;
  }
  @Override
  public NoResult visit(GetMembers e) {
    return e.targhet.accept(this);
  }
  @Override
  public NoResult visit(Try e) {
    visit(e._try);
    for (ClassName cn : e.types)
      visit(cn);
    for (VariableName vn : e.names)
      visit(vn);
    for (Block b : e.blocks)
      visit(b);
    return noResult;
  }
  @Override
  public NoResult visit(Foreach e) {
    visit(e.type);
    visit(e.name);
    e.expression.accept(this);
    e.body.accept(this);
    return noResult;
  }
}