package visitors;

import java.util.Collection;
import java.util.Iterator;

import structure.astNode.*;
import structure.astNode.Ast.*;
import structure.astNode.Ast.StringValue;

public class VisitorToAstJavaCode extends VisitorReader {
  StringBuffer result = new StringBuffer();
  public static StringBuffer run(Ast node) {
    VisitorToAstJavaCode v = new VisitorToAstJavaCode();
    node.accept(v);
    return v.result;
  }
  private void c(Object o) {
    this.result.append(o);
    this.result.append(" ");
  }
  
  public void nativeRepr(String o) {
    this.result.append("\"");
    this.result.append(VisitorTranslateZero.javaEscapeString(o));
    this.result.append("\"");
  }
  public void nativeRepr(int o) {
    this.result.append(o);
  }
  public void nativeRepr(boolean o) {
    this.result.append(o);
  }
  private void seq(Class<?> clazz, String s) {
    c(Sequence.class.getCanonicalName() + "." + s + clazz.getSimpleName());
  }
  private void open(Ast e) {
    this.result.append("new ");
    this.result.append(e.getClass().getCanonicalName());
    this.result.append("( ");
  }
  private NoResult close() {
    c(")");
    return noResult;
  }
  private void comma() {
    c(",");
  }
  private <T extends Ast> void commaSepList(Collection<T> collection) {
    Iterator<T> it = collection.iterator();
    if (!it.hasNext()) {
      c("()");
      return;
    }
    c("(");
    it.next().accept(this);
    while (it.hasNext()) {
      c(",");
      it.next().accept(this);
    }
    c(")");
  }
  @Override
  public NoResult visit(IntValue e) {
    open(e);
    nativeRepr(e.value);
    return close();
  }
  @Override
  public NoResult visit(StringValue e) {
    open(e);
    nativeRepr(e.value);
    return close();
  }
  @Override
  public NoResult visit(BoolValue e) {
    open(e);
    nativeRepr(e.value);
    return close();
  }
  @Override
  public NoResult visit(ClassNameValue e) {
    open(e);
    nativeRepr(e.value);
    return close();
  }
  @Override
  public NoResult visit(MemberNameValue e) {
    open(e);
    nativeRepr(e.value);
    return close();
  }
  @Override
  public NoResult visit(ClassName e) {
    open(e);
    nativeRepr(e.value);
    return close();
  }
  @Override
  public NoResult visit(MemberName e) {
    open(e);
    nativeRepr(e.value);
    return close();
  }
  @Override
  public NoResult visit(VariableName e) {
    open(e);
    nativeRepr(e.value);
    return close();
  }
  @Override
  public NoResult visit(VariableAccess e) {
    open(e);
    visit(e.value);
    return close();
  }
  @Override
  public NoResult visit(InternalFieldAccess e) {
    open(e);
    visit(e.value);
    return close();
  }
  @Override
  public NoResult visit(InternalMethodCall e) {
    open(e);
    visit(e.value);
    comma();
    visit(e.params);
    return close();
  }
  @Override
  public NoResult visit(ClientFieldAccess e) {
    open(e);
    e.reciver.accept(this);
    comma();
    visit(e.value);
    return close();
  }
  @Override
  public NoResult visit(ClientMethodCall e) {
    open(e);
    e.reciver.accept(this);
    comma();
    visit(e.value);
    comma();
    visit(e.params);
    return close();
  }
  @Override
  public NoResult visit(BaseClassValue e) {
    open(e);
    seq(ClassName.class, "hs");
    commaSepList(e.impl);
    comma();
    seq(Member.class, "m");
    commaSepList(e.members.values());
    comma();
    visit(e.constructor);
    return close();
  }
  @Override
  public NoResult visit(ParamsDec e) {
    open(e);
    seq(ClassName.class, "al");
    commaSepList(e.types);
    comma();
    seq(VariableName.class, "al");
    commaSepList(e.names);
    return close();
  }
  @Override
  public NoResult visit(ParamsUse e) {
    open(e);
    seq(Expression.class, "al");
    commaSepList(e.actual);
    return close();
  }
  @Override
  public NoResult visit(Constructor e) {
    open(e);
    e.params.accept(this);
    comma();
    seq(MemberName.class, "al");
    commaSepList(e.names);
    comma();
    seq(Expression.class, "al");
    commaSepList(e.expressions);
    return close();
  }
  @Override
  public NoResult visit(Field e) {
    open(e);
    e.type.accept(this);
    comma();
    e.name.accept(this);
    return close();
  }
  @Override
  public NoResult visit(AbstractField e) {
    open(e);
    visit(e.type);
    comma();
    visit(e.name);
    return close();
  }
  @Override
  public NoResult visit(Method e) {
    open(e);
    visit(e.returnType);
    comma();
    visit(e.name);
    comma();
    visit(e.params);
    comma();
    visit(e.body);
    return close();
  }
  @Override
  public NoResult visit(Block e) {
    open(e);
    seq(Statement.class, "al");
    commaSepList(e.statements);
    return close();
  }
  @Override
  public NoResult visit(If e) {
    open(e);
    e.condition.accept(this);
    comma();
    e.then.accept(this);
    comma();
    e._else.accept(this);
    return close();
  }
  @Override
  public NoResult visit(While e) {
    open(e);
    e.condition.accept(this);
    comma();
    e.body.accept(this);
    return close();
  }
  @Override
  public NoResult visit(VariableDeclaration e) {
    open(e);
    visit(e.type);
    comma();
    visit(e.name);
    comma();
    e.initialization.accept(this);
    return close();
  }
  @Override
  public NoResult visit(Return e) {
    open(e);
    e.expression.accept(this);
    return close();
  }
  @Override
  public NoResult visit(AbstractMethod e) {
    open(e);
    visit(e.returnType);
    comma();
    visit(e.name);
    comma();
    visit(e.params);
    return close();
  }
  @Override
  public NoResult visit(New e) {
    open(e);
    visit(e.type);
    comma();
    visit(e.params);
    return close();
  }
  @Override
  public NoResult visit(Print e) {
    open(e);
    e.expression.accept(this);
    return close();
  }
  @Override
  public NoResult visit(BinOpExp e) {
    open(e);
    e.left.accept(this);
    comma();
    c(BinOp.class.getName());
    c(".");
    c(e.op.name());
    comma();
    e.right.accept(this);
    return close();
  }
  @Override
  public NoResult visit(UnOpExp e) {
    open(e);
    c(UnOp.class.getName());
    c(".");
    c(e.op.name());
    comma();
    e.inner.accept(this);
    return close();
  }
  @Override
  public NoResult visit(RestrictExp e) {
    open(e);
    e.targhet.accept(this);
    comma();
    e.inner.accept(this);
    return close();
  }
  @Override
  public NoResult visit(AliasExp e) {
    open(e);
    e.targhet.accept(this);
    comma();
    e.left.accept(this);
    comma();
    e.right.accept(this);
    return close();
  }
  @Override
  public NoResult visit(RedirectExp e) {
    open(e);
    e.targhet.accept(this);
    comma();
    e.left.accept(this);
    comma();
    e.right.accept(this);
    return close();
  }
  
  @Override
  public NoResult visit(NewArray e) {
    open(e);
    visit(e.type);
    comma();
    visit(e.params);
    return close();
  }
  @Override
  public NoResult visit(StmExp e) {
    open(e);
    e.expression.accept(this);
    return close();
  }
  @Override
  public NoResult visit(VariableAssignament e) {
    open(e);
    visit(e.name);
    comma();
    e.assignedValue.accept(this);
    return close();
    
  }
  @Override
  public NoResult visit(Throw e) {
    open(e);
    e.expression.accept(this);
    return close();
  }
  @Override
  public NoResult visit(AccessArray e) {
    open(e);
    e.targhet.accept(this);
    comma();
    e.inner.accept(this);
    return close();
  }
  @Override
  public NoResult visit(GetMembers e) {
    open(e);
    e.targhet.accept(this);
    return close();
  }
  @Override
  public NoResult visit(Try e) {
    open(e);
    visit(e._try);
    comma();
    seq(ClassName.class, "al");
    commaSepList(e.types);
    comma();
    seq(VariableName.class, "al");
    commaSepList(e.names);
    comma();
    seq(Block.class, "al");
    commaSepList(e.blocks);
    return close();
  }
  @Override
  public NoResult visit(Foreach e) {
    open(e);
    visit(e.type);
    comma();
    visit(e.name);
    comma();
    e.expression.accept(this);
    comma();;
    e.body.accept(this);
    return close();
  }
}
