package visitors;

import java.util.Iterator;

import structure.astNode.Ast;
import structure.astNode.Ast.*;
import structure.astNode.Ast.StringValue;

public class VisitorToString extends VisitorReader {
  StringBuffer result = new StringBuffer();
  public StringBuffer run(Ast node) {
    node.accept(this);
    return this.result;
  }
  protected void c(Object o) {
    this.result.append(o);
    this.result.append(" ");
  }
  private int indentLevel = 0;
  protected void newLine() {
    result.append("\n");
    for (int i = 0; i < indentLevel; i++)
      result.append("   ");
  }
  protected void indent() {
    indentLevel += 1;
    newLine();
  }
  protected void deIndent() {
    indentLevel -= 1;
  }
  @Override
  public NoResult visit(IntValue e) {
    this.c(e.value);
    return noResult;
  }
  @Override
  public NoResult visit(StringValue e) {
    this.c(e.value);
    return noResult;
  }
  @Override
  public NoResult visit(BoolValue e) {
    this.c(e.value);
    return noResult;
  }
  @Override
  public NoResult visit(ClassNameValue e) {
    //this.c("$$");
    this.c(e.value);
    return noResult;
  }
  @Override
  public NoResult visit(MemberNameValue e) {
    this.c("$");
    this.c(e.value);
    return noResult;
  }
  @Override
  public NoResult visit(ClassName e) {
    this.c(e.value);
    return noResult;
  }
  @Override
  public NoResult visit(MemberName e) {
    this.c(e.value);
    return noResult;
  }
  @Override
  public NoResult visit(VariableName e) {
    this.c(e.value);
    return noResult;
  }
  @Override
  public NoResult visit(VariableAccess e) {
    visit(e.value);
    return noResult;
  }
  @Override
  public NoResult visit(InternalFieldAccess e) {
    visit(e.value);
    return noResult;
  }
  @Override
  public NoResult visit(InternalMethodCall e) {
    visit(e.value);
    visit(e.params);
    return noResult;
  }
  @Override
  public NoResult visit(ClientFieldAccess e) {
    e.reciver.accept(this);
    this.c(".");
    visit(e.value);
    return noResult;
  }
  @Override
  public NoResult visit(ClientMethodCall e) {
    e.reciver.accept(this);
    this.c(".");
    e.value.accept(this);
    e.params.accept(this);
    return noResult;
  }
  
  private void indentEmpty(BaseClassValue e) {
    printImplemements(e);
    c("{");
    visit(e.constructor);
    c("}");
  }
  
  @Override
  public NoResult visit(BaseClassValue e) {
    if (e.members.values().isEmpty()) {
      indentEmpty(e);
      return noResult;
    }
    indent();
    printImplemements(e);
    c("{");
    indent();
    for (Member m : e.members.values()) {
      m.accept(this);
      newLine();
    }
    visit(e.constructor);
    deIndent();
    newLine();
    this.c("}");
    deIndent();
    return noResult;
  }
  private void printImplemements(BaseClassValue e) {
    Iterator<ClassName> it = e.impl.iterator();
    if (it.hasNext()) {
      c("implements");
      visit(it.next());
      while (it.hasNext()) {
        c(",");
        visit(it.next());
      }
    }
  }
  @Override
  public NoResult visit(ParamsDec e) {
    this.c("(");
    assert e.names.size() == e.types.size();
    if (e.names.size() > 0) {
      e.types.get(0).accept(this);
      e.names.get(0).accept(this);
    }
    for (int i = 1; i < e.names.size(); i++) {
      this.c(",");
      e.types.get(i).accept(this);
      e.names.get(i).accept(this);
    }
    this.c(")");
    return noResult;
  }
  @Override
  public NoResult visit(ParamsUse e) {
    this.c("(");
    if (e.actual.size() > 0) e.actual.get(0).accept(this);
    for (int i = 1; i < e.actual.size(); i++) {
      this.c(",");
      e.actual.get(i).accept(this);
    }
    this.c(")");
    return noResult;
  }
  public NoResult visitAlternate(ParamsUse e) {
    this.c("{");
    if (e.actual.size() > 0) e.actual.get(0).accept(this);
    for (int i = 1; i < e.actual.size(); i++) {
      this.c(",");
      e.actual.get(i).accept(this);
    }
    this.c("}");
    return noResult;
  }
  @Override
  public NoResult visit(Constructor e) {
    this.c("constructor");
    e.params.accept(this);
    this.c("{");
    if (e.names.size() > 1) indent();
    assert e.names.size() == e.expressions.size();
    for (int i = 0; i < e.names.size(); i++) {
      e.names.get(i).accept(this);
      this.c("=");
      e.expressions.get(i).accept(this);
      this.c(";");
      if (e.names.size() > 1) newLine();
    }
    if (e.names.size() > 1) deIndent();
    this.c("}");
    return noResult;
  }
  @Override
  public NoResult visit(Field e) {
    visit(e.type);
    visit(e.name);
    this.c(";");
    return noResult;
  }
  @Override
  public NoResult visit(AbstractField e) {
    this.c("abstract");
    visit(e.type);
    visit(e.name);
    this.c(";");
    return noResult;
  }
  @Override
  public NoResult visit(Method e) {
    visit(e.returnType);
    visit(e.name);
    visit(e.params);
    visit(e.body);
    return noResult;
  }
  
  private boolean haveToIndent(Block e) {
    return e.statements.size() > 1 //
        || ((!e.statements.isEmpty()) && //
        (//
        e.statements.get(0) instanceof If //
        || e.statements.get(0) instanceof While)//
        );
  }
  @Override
  public NoResult visit(Block e) {
    this.c("{");
    if (haveToIndent(e)) indent();
    for (Statement s : e.statements) {
      s.accept(this);
      if (haveToIndent(e)) newLine();
    }
    if (haveToIndent(e)) deIndent();
    this.c("}");
    return noResult;
  }
  @Override
  public NoResult visit(If e) {
    this.c("if (");
    e.condition.accept(this);
    this.c(")");
    e.then.accept(this);
    newLine();
    this.c("else");
    e._else.accept(this);
    return noResult;
  }
  @Override
  public NoResult visit(While e) {
    this.c("while (");
    e.condition.accept(this);
    this.c(")");
    e.body.accept(this);
    return noResult;
  }
  @Override
  public NoResult visit(VariableDeclaration e) {
    visit(e.type);
    visit(e.name);
    this.c("=");
    e.initialization.accept(this);
    this.c(";");
    return noResult;
  }
  @Override
  public NoResult visit(Return e) {
    this.c("return");
    e.expression.accept(this);
    this.c(";");
    return noResult;
  }
  @Override
  public NoResult visit(AbstractMethod e) {
    this.c("abstract");
    visit(e.returnType);
    visit(e.name);
    visit(e.params);
    this.c(";");
    return noResult;
  }
  @Override
  public NoResult visit(New e) {
    this.c("new");
    visit(e.type);
    visit(e.params);
    return noResult;
  }
  @Override
  public NoResult visit(Print e) {
    this.c("print (");
    e.expression.accept(this);
    this.c(") ;");
    return noResult;
  }
  @Override
  public NoResult visit(BinOpExp e) {
    this.c("(");
    e.left.accept(this);
    this.c(e.op.repr());
    e.right.accept(this);
    this.c(")");
    return noResult;
  }
  @Override
  public NoResult visit(UnOpExp e) {
    this.c(e.op.repr());
    e.inner.accept(this);
    return noResult;
  }
  @Override
  public NoResult visit(RestrictExp e) {
    e.targhet.accept(this);
    this.c("[ restrict");
    e.inner.accept(this);
    this.c("]");
    return noResult;
  }
  @Override
  public NoResult visit(AliasExp e) {
    e.targhet.accept(this);
    this.c("[");
    this.c("alias");
    e.left.accept(this);
    this.c("to");
    e.right.accept(this);
    this.c("]");
    return noResult;
  }
  @Override
  public NoResult visit(RedirectExp e) {
    e.targhet.accept(this);
    this.c("[");
    this.c("redirect");
    e.left.accept(this);
    this.c("to");
    e.right.accept(this);
    this.c("]");
    return noResult;
  }
  
  @Override
  public NoResult visit(NewArray e) {
    c("new");
    visit(e.type);
    c("[]");
    visitAlternate(e.params);
    return noResult;
  }
  @Override
  public NoResult visit(StmExp e) {
    e.expression.accept(this);
    c(";");
    return noResult;
  }
  @Override
  public NoResult visit(VariableAssignament e) {
    visit(e.name);
    c("=");
    e.assignedValue.accept(this);
    c(";");
    return noResult;
    
  }
  @Override
  public NoResult visit(Throw e) {
    c("throw");
    e.expression.accept(this);
    c(";");
    return noResult;
  }
  @Override
  public NoResult visit(AccessArray e) {
    e.targhet.accept(this);
    c("[");
    e.inner.accept(this);
    c("]");
    return noResult;
  }
  @Override
  public NoResult visit(GetMembers e) {
    e.targhet.accept(this);
    c("[");
    c("members");
    c("]");
    return noResult;
  }
  @Override
  public NoResult visit(Try e) {
    c("try");
    visit(e._try);
    assert (e.types.size() == e.names.size()) && (e.types.size() == e.blocks.size());
    for (int i = 0; i < e.types.size(); i++) {
      newLine();
      c("catch");
      c("(");
      visit(e.types.get(i));
      visit(e.names.get(i));
      c(")");
      visit(e.blocks.get(i));
    }
    return noResult;
  }
  @Override
  public NoResult visit(Foreach e) {
    c("for (");
    visit(e.type);
    visit(e.name);
    c(":");
    e.expression.accept(this);
    c(")\n");
    e.body.accept(this);
    return noResult;
  }
}
