package visitors;

import static tools.DataStructures.*;

import java.util.*;

import primitiveType.*;
import primitiveType.PredefinedExceptions.*;
import structure.astNode.BinOp;
import structure.astNode.Ast.*;
import structure.integrity.IntegrityChecker;
import structure.program.ClassTable;
import structure.resource.GreenResource;
import structure.resource.YellowResource.NormalizedYellowResource;

import compiler.Mfjc;

public class VisitorTranslateZero extends VisitorToString {
  public static final String thisBaseClassValue = "01ThisBaseClassValue";
  public static int generationNumber = 0;
  
  public static final HashMap<Character, Character> escapeSymbols = M(//
      T('\\', '\\'),//
      T('\b', 'b'),//
      T('\t', 't'),//
      T('\n', 'n'),//
      T('\f', 'f'),//
      T('\r', 'r'),//
      T('\'', '\''),//
      T('\"', '\"'),//
      T('\0', '0')//
  );
  
  public static String javaEscapeString(String s) {
    char[] start = s.toCharArray();
    ArrayList<Character> result = new ArrayList<Character>();
    for (char c : start) {
      if (escapeSymbols.keySet().contains(c)) {
        result.add('\\');
        result.add(escapeSymbols.get(c));
      } else result.add(c);
    }
    return new String(tools.DataStructures.toArrayBase(char[].class, char.class, result));
  }
  
  public static String iName(String e) {
    return Mfjc.programCode + ".i" + generationNumber + e;
  }
  public static String cName(String e) {
    assert e != null;
    return Mfjc.programCode + ".c" + generationNumber + e;
  }
  public static String cDecName(String e) {
    assert e != null;
    return "c" + generationNumber + e;
  }
  public static String iDecName(String e) {
    assert e != null;
    return "i" + generationNumber + e;
  }
  public static String nTilde(MemberNameValue e) {
    return "new " + MFJName.class.getCanonicalName() + "(\"" + javaEscapeString(e.value) + "\")";
  }
  private int countGeneratedConstants = 0;
  private HashMap<String, StringBuffer> generatedConstants = new HashMap<String, StringBuffer>();
  private String thisTypeName;
  public VisitorTranslateZero(String thisTypeName) {
    this.thisTypeName = thisTypeName;
  }
  
  public static boolean isThrowable(ClassTable ct, BaseClassValue bc) {
    //if (bc.impl.contains(IntegrityChecker.runtimeExceptionClassName)) return true;
    for (ClassName cn : IntegrityChecker.predefinedThrowables)
      if (bc.impl.contains(cn)) return true;
    for (ClassName cn : bc.impl) {
      if (ct.keySet(GreenResource.class).contains(cn) && isThrowable(ct, ct.get(GreenResource.class, cn).getValue())) return true;
      if (ct.keySet(NormalizedYellowResource.class).contains(cn) && isThrowable(ct, ct.get(NormalizedYellowResource.class, cn).expression))
        return true;
    }
    return false;
  }
  public StringBuffer runBaseClassValue(ClassTable ct, BaseClassValue bc) {
    if (bc.members.values(IAbstract.class).isEmpty()) c("public class");
    else c("public abstract class");
    c(cDecName(thisTypeName));
    if (isThrowable(ct, bc)) c("extends RuntimeException ");
    c("implements");
    c(iName(thisTypeName));
    c("{\n");
    visit(bc.constructor);
    c("\n");
    for (Member m : bc.members.values()) {
      m.accept(this);
      c("\n");
    }
    generateConstants();
    StringBuffer thisAst = VisitorToAstJavaCode.run(bc);
    StringBuffer start = new StringBuffer("public static " + BaseClassValue.class.getCanonicalName() + " " + IntegrityChecker.forbiddenPrefix
        + thisBaseClassValue + " = ");
    c(start);
    c(thisAst);
    c(";");
    c("\n}");
    return this.result;
  }
  public void generateConstants() {
    for (String s : generatedConstants.keySet()) {
      c("private static " + BaseClassValue.class.getCanonicalName() + " " + s + " = " + generatedConstants.get(s) + ";\n");
    }
  }
  public StringBuffer runInterface(BaseClassValue bc) {
    c("public interface");
    c(iDecName(thisTypeName));
    HashSet<ClassName> localImpl = new HashSet<ClassName>(bc.impl);
    localImpl.remove(IntegrityChecker.objectClassName);
    if (localImpl.contains(IntegrityChecker.runtimeExceptionClassName)) {
      localImpl.remove(IntegrityChecker.runtimeExceptionClassName);
    }
    Iterator<ClassName> cI = localImpl.iterator();
    if (cI.hasNext()) {
      c("extends");
      visit(cI.next());
      while (cI.hasNext()) {
        c(",");
        visit(cI.next());
      }
    }
    c("{\n");
    for (Field m : bc.members.values(Field.class)) {
      visit(m.type);
      visit(m.name);
      c("();\n");
    }
    for (AbstractField m : bc.members.values(AbstractField.class)) {
      visit(m.type);
      visit(m.name);
      c("();\n");
    }
    for (Method m : bc.members.values(Method.class)) {
      visit(m.returnType);
      visit(m.name);
      visit(m.params);
      c(";\n");
    }
    for (AbstractMethod m : bc.members.values(AbstractMethod.class)) {
      visit(m.returnType);
      visit(m.name);
      visit(m.params);
      c(";\n");
    }
    generateConstants();
    c("\n}");
    return this.result;
  }
  public String cTilde(BaseClassValue e) {
    String memberName = IntegrityChecker.forbiddenPrefix + (countGeneratedConstants++);
    generatedConstants.put(memberName, VisitorToAstJavaCode.run(e));
    return "new " + MFJClass.class.getCanonicalName() + "(" + memberName + ")";
  }
  
  @Override
  public NoResult visit(ClassNameValue e) {
    c(" new ");
    c(MFJClass.class.getCanonicalName());
    c("(");
    c(cName(e.value) + "." + IntegrityChecker.forbiddenPrefix + thisBaseClassValue);
    c(")");
    return noResult;
  }
  @Override
  public NoResult visit(MemberNameValue e) {
    c(nTilde(e));
    return noResult;
  }
  
  String getNudeType(String type) {
    int i = type.indexOf('[');
    if (i == -1) return type;
    return type.substring(0, i);
  }
  String getArraySquares(String type) {
    int i = type.indexOf('[');
    if (i == -1) return "";
    return type.substring(i);
  }
  String iSpecialName(String name) {
    if (name.equals("class")) return MFJClass.class.getCanonicalName();
    if (name.equals("name")) return MFJName.class.getCanonicalName();
    if (name.equals(CompositionException.class.getSimpleName())) //
      return PredefinedExceptions.ICompositionException.class.getCanonicalName();
    if (name.equals(RestrictException.class.getSimpleName())) //
      return PredefinedExceptions.IRestrictException.class.getCanonicalName();
    if (name.equals(AliasException.class.getSimpleName())) //
      return PredefinedExceptions.IAliasException.class.getCanonicalName();
    if (name.equals(RedirectException.class.getSimpleName())) //
      return PredefinedExceptions.IRedirectException.class.getCanonicalName();
    if (name.equals(SumException.class.getSimpleName())) //
      return PredefinedExceptions.ISumException.class.getCanonicalName();
    return name;
  }
  String cSpecialName(String name) {
    if (name.equals(CompositionException.class.getSimpleName())) //
      return CompositionException.class.getCanonicalName();
    if (name.equals(RestrictException.class.getSimpleName())) //
      return RestrictException.class.getCanonicalName();
    if (name.equals(AliasException.class.getSimpleName())) //
      return AliasException.class.getCanonicalName();
    if (name.equals(RedirectException.class.getSimpleName())) //
      return RedirectException.class.getCanonicalName();
    if (name.equals(SumException.class.getSimpleName())) //
      return SumException.class.getCanonicalName();
    return name;
  }
  @Override
  public NoResult visit(ClassName e) {
    String nudeType = getNudeType(e.value);
    if (IntegrityChecker.predefinedTypeAndClassString.contains(nudeType)) {
      nudeType = iSpecialName(nudeType);
      c(nudeType + getArraySquares(e.value));
    } else c(iName(e.value));
    return noResult;
  }
  @Override
  public NoResult visit(BaseClassValue e) {
    c(cTilde(e));
    return noResult;
  }
  @Override
  public NoResult visit(Constructor e) {
    c("public");
    c(cDecName(thisTypeName));
    e.params.accept(this);
    this.c("{");
    assert e.names.size() == e.expressions.size();
    for (int i = 0; i < e.names.size(); i++) {
      e.names.get(i).accept(this);
      this.c("=");
      e.expressions.get(i).accept(this);
      this.c(";");
    }
    this.c("}");
    return noResult;
  }
  @Override
  public NoResult visit(Field e) {
    c("private");
    visit(e.type);
    visit(e.name);
    this.c(";");
    c("public");
    visit(e.type);
    visit(e.name);
    c("(){return ");
    visit(e.name);
    c(";}\n");
    return noResult;
  }
  @Override
  public NoResult visit(AbstractField e) {
    c("public");
    c("abstract");
    visit(e.type);
    visit(e.name);
    c("();");
    return noResult;
  }
  @Override
  public NoResult visit(Method e) {
    c("public");
    return super.visit(e);
  }
  @Override
  public NoResult visit(AbstractMethod e) {
    c("public");
    return super.visit(e);
  }
  @Override
  public NoResult visit(New e) {//NewArray is CORRECTLY not overridden
    this.c("new");
    String nudeType = getNudeType(e.type.value);
    if (IntegrityChecker.predefinedTypeAndClassString.contains(nudeType)) {
      nudeType = cSpecialName(nudeType);
      assert getArraySquares(e.type.value).equals("");
      c(nudeType);
    } else c(cName(e.type.value));
    visit(e.params);
    return noResult;
  }
  @Override
  public NoResult visit(BinOpExp e) {
    if (e.op.equals(BinOp.ClassSum)) {
      e.left.accept(this);
      c(".__operatorSum(");
      e.right.accept(this);
      c(")");
      return noResult;
    }
    if (e.op.equals(BinOp.Equals)) {
      c(MFJHelper.class.getCanonicalName());
      c(".__areEquals(");
      e.left.accept(this);
      c(",");
      e.right.accept(this);
      c(")");
      return noResult;
    }
    return super.visit(e);
    
  }
  @Override
  public NoResult visit(RestrictExp e) {
    e.targhet.accept(this);
    c(".__operatorRestrict(");
    e.inner.accept(this);
    c(")");
    return noResult;
  }
  @Override
  public NoResult visit(AliasExp e) {
    e.targhet.accept(this);
    c(".__operatorAlias(");
    e.left.accept(this);
    this.c(",");
    e.right.accept(this);
    this.c(")");
    return noResult;
  }
  @Override
  public NoResult visit(RedirectExp e) {
    e.targhet.accept(this);
    c(".__operatorRedirect(");
    e.left.accept(this);
    this.c(",");
    e.right.accept(this);
    this.c(")");
    return noResult;
  }
  @Override
  public NoResult visit(InternalFieldAccess e) {
    super.visit(e);
    c("()");
    return noResult;
  }
  @Override
  public NoResult visit(ClientFieldAccess e) {
    super.visit(e);
    c("()");
    return noResult;
  }
  
  @Override
  public NoResult visit(Print e) {
    c(Mfjc.class.getCanonicalName() + ".print(");
    e.expression.accept(this);
    c(") ;");
    return noResult;
  }
  
  @Override
  public NoResult visit(Throw e) {
    c("throw (RuntimeException)");
    e.expression.accept(this);
    c(";");
    return noResult;
  }
  @Override
  public NoResult visit(GetMembers e) {
    e.targhet.accept(this);
    c(".__operatorMembers()");
    return noResult;
  }
  
  private static int countCathVar = 0;
  @Override
  public NoResult visit(Try e) {
    c("try");
    String auxName = IntegrityChecker.forbiddenPrefix + "00" + (countCathVar++);
    visit(e._try);
    assert (e.types.size() == e.names.size()) && (e.types.size() == e.blocks.size());
    c("catch(RuntimeException " + auxName + ") {");
    for (int i = 0; i < e.types.size(); i++) {
      c("if(" + auxName + " instanceof ");
      visit(e.types.get(i));
      c("){\n");
      visit(e.types.get(i));
      c(" ");
      visit(e.names.get(i));
      c("=(");
      visit(e.types.get(i));
      c(")" + auxName + ";\n");
      visit(e.blocks.get(i));
      c("\n} else \n");
    }
    c("throw " + auxName + ";\n}");
    return noResult;
  }
  
}