package visitors;

import java.util.*;

import structure.astNode.Members;
import structure.astNode.Ast.*;
import structure.integrity.ClassNotWellFormed.VariableAlreadyDefined;
import tools.Assertions;
import tools.DataStructures.WrapperException;
public class VisitorTurnFreeVariablesInClassNameValueZero extends VisitorClone {
  HashSet<VariableName> defined;
  static final VariableName thisVarName = new VariableName("this");
  
  public static Expression run(Expression e) {
    VisitorTurnFreeVariablesInClassNameValueZero vv = new VisitorTurnFreeVariablesInClassNameValueZero(new HashSet<VariableName>());
    return (Expression) e.accept(vv);
  }
  public static Statement run(Statement e) {
    VisitorTurnFreeVariablesInClassNameValueZero vv = new VisitorTurnFreeVariablesInClassNameValueZero(new HashSet<VariableName>());
    return (Statement) e.accept(vv);
  }
  
  public static BaseClassValue run(BaseClassValue e) /*throws RecognitionException */{
    try {
      Members members = new Members();
      boolean modMembers = false;
      VisitorTurnFreeVariablesInClassNameValueZero vv = new VisitorTurnFreeVariablesInClassNameValueZero();
      for (Member m : e.members.values()) {
        Member m2 = (Member) m.accept(vv);
        if (!m2.equals(m)) modMembers = true;
        members.put(m2.getName(), m2);
      }
      Constructor c = vv.visit(e.constructor);
      if (c.equals(e.constructor)) c = e.constructor;
      if (!modMembers && c == e.constructor) return e;
      return new BaseClassValue(e.impl, members, c);
    } catch (WrapperException t) {
      //throw (RecognitionException) t.wrappedElement;
      throw Assertions.codeNotReachable("ill formed class:: variable defined twice");
    }
  }
  
  @Override
  public BaseClassValue visit(BaseClassValue e) {
    return e;
  }
  private VisitorTurnFreeVariablesInClassNameValueZero(Collection<VariableName> defined) {
    super();
    this.defined = new HashSet<VariableName>(defined);
  }
  private VisitorTurnFreeVariablesInClassNameValueZero() {
    super();
    this.defined = null;
  }
  
  public Constructor superVisit(Constructor e) {
    return super.visit(e);
  }
  @Override
  public Constructor visit(Constructor e) {
    return new VisitorTurnFreeVariablesInClassNameValueZero(//
        e.params.names).superVisit(e);
  }
  public Method superVisit(Method e) {
    return super.visit(e);
  }
  @Override
  public Method visit(Method e) {
    HashSet<VariableName> def = new HashSet<VariableName>(e.params.names);
    def.add(thisVarName);
    return new VisitorTurnFreeVariablesInClassNameValueZero(//
        def).superVisit(e);
  }
  
  public Block superVisit(Block e) {
    return super.visit(e);
  }
  @Override
  public Block visit(Block e) {
    return new VisitorTurnFreeVariablesInClassNameValueZero(//
        defined).superVisit(e);
  }
  
  @Override
  public VariableDeclaration visit(VariableDeclaration e) {
    if (defined.contains(e.name)) throw new WrapperException(new VariableAlreadyDefined(e.name.value));
    defined.add(e.name);
    return super.visit(e);
  }
  @Override
  public Expression visit(VariableAccess e) {
    if (!defined.contains(e.value)) return new ClassNameValue(e.value.value);
    return super.visit(e);
  }
  
  @Override
  public Try visit(Try e) {
    Block _try = visit(e._try);
    ArrayList<Block> blocks = new ArrayList<Block>();
    assert (e.types.size() == e.names.size()) && (e.types.size() == e.blocks.size());
    for (int i = 0; i < e.types.size(); i++) {
      if (defined.contains(e.names.get(i))) throw new WrapperException(new VariableAlreadyDefined(e.names.get(i).value));
      HashSet<VariableName> def = new HashSet<VariableName>(defined);
      def.add(e.names.get(i));
      blocks.add(new VisitorTurnFreeVariablesInClassNameValueZero(//
          def).superVisit(e.blocks.get(i)));
    }
    if (_try.equals(e._try) && blocks.equals(e.blocks)) return e;
    return new Try(_try, e.types, e.names, blocks);
  }
  @Override
  public Foreach visit(Foreach e) {
    Expression expression = (Expression) e.expression.accept(this);
    if (defined.contains(e.name)) throw new WrapperException(new VariableAlreadyDefined(e.name.value));
    HashSet<VariableName> def = new HashSet<VariableName>(defined);
    def.add(e.name);
    Statement body = (Statement) e.body.accept(new VisitorTurnFreeVariablesInClassNameValueZero(def));
    if (expression.equals(e.expression) && body.equals(e.body)) return e;
    return new Foreach(e.type, e.name, expression, body);
  }
}
